@@ -241,7 +241,7 @@ extern "C" {
 // basic funcs
 
 s32  CALLBACK GSinit();
-s32  CALLBACK GSopen(void *pDsp, char *Title, int multithread);
+s32  CALLBACK GSopen(void *pDsp, const char *Title, int multithread);
 void CALLBACK GSclose();
 void CALLBACK GSshutdown();
 void CALLBACK GSsetSettingsDir( const char* dir );
@@ -572,7 +572,7 @@ typedef void (CALLBACK* _PS2EsetEmuVersion)(const char* emuId, u32 version);		//
 // GS
 // NOTE: GSreadFIFOX/GSwriteCSR functions CANNOT use XMM/MMX regs
 // If you want to use them, need to save and restore current ones
-typedef s32  (CALLBACK* _GSopen)(void *pDsp, char *Title, int multithread);
+typedef s32  (CALLBACK* _GSopen)(void *pDsp, const char *Title, int multithread);
 typedef s32  (CALLBACK* _GSopen2)( void *pDsp, u32 flags );
 typedef void (CALLBACK* _GSvsync)(int field);
 typedef void (CALLBACK* _GSgifTransfer)(const u32 *pMem, u32 size);
@@ -702,6 +702,7 @@ typedef void (CALLBACK* _DEV9writeDMA8Mem)(u32 *pMem, int size);
 #endif
 typedef void (CALLBACK* _DEV9irqCallback)(DEV9callback callback);
 typedef DEV9handler (CALLBACK* _DEV9irqHandler)(void);
+typedef void (CALLBACK* _DEV9async)(u32 cycles);
 
 // USB
 // NOTE: The read/write functions CANNOT use XMM/MMX regs
@@ -820,6 +821,7 @@ extern _DEV9writeDMA8Mem  DEV9writeDMA8Mem;
 #endif
 extern _DEV9irqCallback   DEV9irqCallback;
 extern _DEV9irqHandler    DEV9irqHandler;
+extern _DEV9async         DEV9async;
 
 // USB
 extern _USBopen           USBopen;
