@@ -32,6 +32,15 @@ static DeclareTls(ConsoleColors) conlog_Color( DefaultConsoleColor );
 static wxString	m_buffer;		// used by ConsoleBuffer
 static Mutex	m_bufferlock;	// used by ConsoleBuffer
 
+#ifdef __linux__
+static FILE *stdout_fp = stdout;
+
+void Console_SetStdout(FILE *fp)
+{
+	stdout_fp = fp;
+}
+#endif
+
 // This function re-assigns the console log writer(s) to the specified target.  It makes sure
 // to flush any contents from the buffered console log (which typically accumulates due to
 // log suspension during log file/window re-init operations) into the new log.
@@ -55,11 +64,11 @@ void Console_SetActiveHandler( const IConsoleWriter& writer, FILE* flushfp )
 			writer.DoWriteLn( ConsoleBuffer_Get() );
 	}
 
-	const_cast<IConsoleWriter&>(Console)		= writer;
-	const_cast<IConsoleWriter&>(DevConWriter)	= writer;
+	Console			= writer;
+	DevConWriter	= writer;
 
 #ifdef PCSX2_DEBUG
-	const_cast<IConsoleWriter&>(DbgCon)	= writer;
+	DbgCon	= writer;
 #endif
 }
 
@@ -71,12 +80,12 @@ void MSW_OutputDebugString( const wxString& text )
 	static bool hasDebugger = wxIsDebuggerRunning();
 	if( hasDebugger ) OutputDebugString( text );
 #else
-	// send them to stderr
-	wxPrintf(L"%s", text.c_str());
-	fflush(stderr);
+	fputs(text.utf8_str(), stdout_fp);
+	fflush(stdout_fp);
 #endif
 }
 
+
 // --------------------------------------------------------------------------------------
 //  ConsoleNull
 // --------------------------------------------------------------------------------------
@@ -105,43 +114,43 @@ const IConsoleWriter ConsoleWriter_Null =
 // --------------------------------------------------------------------------------------
 
 #ifdef __linux__
-static __fi const wxChar* GetLinuxConsoleColor(ConsoleColors color)
+static __fi const char* GetLinuxConsoleColor(ConsoleColors color)
 {
     switch(color)
     {
         case Color_Black:
-        case Color_StrongBlack: return L"\033[30m\033[1m";
+        case Color_StrongBlack: return "\033[30m\033[1m";
 
-        case Color_Red: return L"\033[31m";
-        case Color_StrongRed: return L"\033[31m\033[1m";
+        case Color_Red: return "\033[31m";
+        case Color_StrongRed: return "\033[31m\033[1m";
 
-        case Color_Green: return L"\033[32m";
-        case Color_StrongGreen: return L"\033[32m\033[1m";
+        case Color_Green: return "\033[32m";
+        case Color_StrongGreen: return "\033[32m\033[1m";
 
-        case Color_Yellow: return L"\033[33m";
-        case Color_StrongYellow: return L"\033[33m\033[1m";
+        case Color_Yellow: return "\033[33m";
+        case Color_StrongYellow: return "\033[33m\033[1m";
 
-        case Color_Blue: return L"\033[34m";
-        case Color_StrongBlue: return L"\033[34m\033[1m";
+        case Color_Blue: return "\033[34m";
+        case Color_StrongBlue: return "\033[34m\033[1m";
 
         // No orange, so use magenta.
         case Color_Orange:
-        case Color_Magenta: return L"\033[35m";
+        case Color_Magenta: return "\033[35m";
         case Color_StrongOrange:
-        case Color_StrongMagenta: return L"\033[35m\033[1m";
+        case Color_StrongMagenta: return "\033[35m\033[1m";
 
-        case Color_Cyan: return L"\033[36m";
-        case Color_StrongCyan: return L"\033[36m\033[1m";
+        case Color_Cyan: return "\033[36m";
+        case Color_StrongCyan: return "\033[36m\033[1m";
 
         // Use 'white' instead of grey.
         case Color_Gray:
-        case Color_White: return L"\033[37m";
+        case Color_White: return "\033[37m";
         case Color_StrongGray:
-        case Color_StrongWhite: return L"\033[37m\033[1m";
+        case Color_StrongWhite: return "\033[37m\033[1m";
 
         // On some other value being passed, clear any formatting.
         case Color_Default:
-        default: return L"\033[0m";
+        default: return "\033[0m";
     }
 }
 #endif
@@ -166,16 +175,17 @@ static void __concall ConsoleStdout_Newline()
 static void __concall ConsoleStdout_DoSetColor( ConsoleColors color )
 {
 #ifdef __linux__
-	wxPrintf(L"\033[0m");
-	wxPrintf(GetLinuxConsoleColor(color));
-	fflush(stdout);
+	fprintf(stdout_fp, "\033[0m%s", GetLinuxConsoleColor(color));
+	fflush(stdout_fp);
 #endif
 }
 
 static void __concall ConsoleStdout_SetTitle( const wxString& title )
 {
 #ifdef __linux__
-	wxPrintf(L"\033]0;%s\007", title.c_str());
+	fputs("\033]0;", stdout_fp);
+	fputs(title.utf8_str(), stdout_fp);
+	fputs("\007", stdout_fp);
 #endif
 }
 
@@ -558,7 +568,10 @@ ConsoleIndentScope::ConsoleIndentScope( int tabs )
 
 ConsoleIndentScope::~ConsoleIndentScope() throw()
 {
-	LeaveScope();
+	try {
+		LeaveScope();
+	}
+	DESTRUCTOR_CATCHALL
 }
 
 void ConsoleIndentScope::EnterScope()
@@ -581,8 +594,11 @@ ConsoleAttrScope::ConsoleAttrScope( ConsoleColors newcolor, int indent )
 
 ConsoleAttrScope::~ConsoleAttrScope() throw()
 {
-	Console.SetColor( m_old_color );
-	Console.SetIndent( -m_tabsize );
+	try {
+		Console.SetColor( m_old_color );
+		Console.SetIndent( -m_tabsize );
+	}
+	DESTRUCTOR_CATCHALL
 }
 
 
@@ -597,15 +613,15 @@ ConsoleAttrScope::~ConsoleAttrScope() throw()
 // 
 #define _DefaultWriter_	ConsoleWriter_Stdout
 
-const IConsoleWriter	Console				= _DefaultWriter_;
-const IConsoleWriter	DevConWriter		= _DefaultWriter_;
+IConsoleWriter	Console				= _DefaultWriter_;
+IConsoleWriter	DevConWriter		= _DefaultWriter_;
 bool					DevConWriterEnabled	= false;
 
 #ifdef PCSX2_DEBUG
-const IConsoleWriter	DbgConWriter		= _DefaultWriter_;
+IConsoleWriter	DbgConWriter		= _DefaultWriter_;
 #endif
 
-const NullConsoleWriter	NullCon = {};
+NullConsoleWriter	NullCon = {};
 
 // --------------------------------------------------------------------------------------
 //  ConsoleLogSource  (implementations)
