@@ -569,32 +569,22 @@ const wxCursor& MoreStockCursors::GetArrowWait()
 MoreStockCursors StockCursors;
 
 // --------------------------------------------------------------------------------------
-//  pxFormatToolTipText / pxSetToolTip
+//  pxSetToolTip
 // --------------------------------------------------------------------------------------
-// This is the preferred way to assign tooltips to wxWindow-based objects, as it performs the
-// necessary text wrapping on platforms that need it.  On windows tooltips are wrapped at 600
-// pixels, or 66% of the screen width, whichever is smaller.  GTK and MAC perform internal
-// wrapping, so this function does a regular assignment there.
-
-wxString pxFormatToolTipText( wxWindow* wind, const wxString& src )
-{
-	// Windows needs manual tooltip word wrapping (sigh).
-	// GTK and Mac are a wee bit more clever (except in GTK tooltips don't show at all
-	// half the time because of some other bug .. sigh)
-
-#ifdef __WXMSW__
-	if( wind == NULL ) return src;		// Silently ignore nulls
-	int whee = wxGetDisplaySize().GetWidth() * 0.75;
-	return pxTextWrapper().Wrap( *wind, src, std::min( whee, 600 ) ).GetResult();
-#else
-	return src;
-#endif
-}
+// This is the preferred way to assign tooltips to wxWidgets-based objects. On Windows it
+// extends the tooltip time to the maximum possible. GTK seems to have indefinite
+// tooltips, I don't know about OS X.
 
 void pxSetToolTip( wxWindow* wind, const wxString& src )
 {
 	if( wind == NULL ) return;		// Silently ignore nulls
-	wind->SetToolTip( pxFormatToolTipText(wind, src) );
+	wind->SetToolTip(src);
+
+	// Make tooltips show for as long as possible on Windows. Linux (GTK) can
+	// show tooltips indefinitely.
+#ifdef __WXMSW__
+	wind->GetToolTip()->SetAutoPop(32767);
+#endif
 }
 
 void pxSetToolTip( wxWindow& wind, const wxString& src )
