@@ -28,25 +28,6 @@ void x86capabilities::CountLogicalCores()
 	LogicalCores = wxThread::GetCPUCount();
 }
 
-bool CanEmitShit()
-{
-	// In Linux I'm pretty sure TLS always works, none of the funny business that Windows
-	// has involving DLLs. >_<
-	return true;
-}
-
-bool CanTestInstructionSets()
-{
-	// Not implemented yet for linux.  (see cpudetect_internal.h for details)
-	return false;
-}
-
-bool _test_instruction( void* pfnCall )
-{
-	// Not implemented yet for linux.  (see cpudetect_internal.h for details)
-	return false;
-}
-
 // Not implemented yet for linux (see cpudetect_internal.h for details)
 SingleCoreAffinity::SingleCoreAffinity() {}
 SingleCoreAffinity::~SingleCoreAffinity() throw() {}
