@@ -107,16 +107,7 @@ Comments) 1950 to 1952 in the files http://tools.ietf.org/html/rfc1950
 #include <zlib/zlib.h>
 #endif
 
-#include <Pcsx2Types.h>
-#ifdef WIN32
-#	define PX_fseeko _fseeki64
-#   define PX_ftello _ftelli64
-#	define PX_off_t  s64        /* __int64 */
-#else
-#	define PX_fseeko fseeko
-#   define PX_ftello ftello
-#	define PX_off_t  off_t
-#endif
+#include "CompressedFileReaderUtils.h"
 
 #define local static
 
@@ -312,6 +303,11 @@ local int build_index(FILE *in, PX_off_t span, struct access **built)
         }
     } while (ret != Z_STREAM_END);
 
+    if (index == NULL) {
+        // Could happen if the start of the stream in Z_STREAM_END
+        return 0;
+    }
+
     /* clean up and return index (release unused entries in list) */
     (void)inflateEnd(&strm);
     index->list = (Point*)realloc(index->list, sizeof(struct point) * index->have);
@@ -336,6 +332,10 @@ typedef struct zstate {
     int isValid;
 } Zstate;
 
+static inline PX_off_t getInOffset(zstate *state) {
+	return state->in_offset;
+}
+
 /* Use the index to read len bytes from offset into buf, return bytes read or
    negative for error (Z_DATA_ERROR or Z_MEM_ERROR).  If data is requested past
    the end of the uncompressed data, then extract() will return a value less
