@@ -34,7 +34,8 @@ enum PluginsEnum_t
 	// time.  So it's placed afer PluginId_Count so that it doesn't show up in the conf
 	// screens or other plugin tables.
 
-	PluginId_Mcd
+	PluginId_Mcd,
+	PluginId_AllocCount // Extra value for correct array allocation
 };
 
 enum GamefixId
@@ -386,7 +387,7 @@ struct Pcsx2Config
 				vuThread        :1;		// Enable Threaded VU1
 		BITFIELD_END
 
-		u8	EECycleRate;		// EE cycle rate selector (1.0, 1.5, 2.0)
+		s8	EECycleRate;		// EE cycle rate selector (1.0, 1.5, 2.0)
 		u8	VUCycleSteal;		// VU Cycle Stealer factor (0, 1, 2, or 3)
 
 		SpeedhackOptions();
@@ -413,13 +414,16 @@ struct Pcsx2Config
 
 		u8 FontWidth;
 		u8 FontHeight;
+		u32 WindowWidth;
+		u32 WindowHeight;
 
 		DebugOptions();
 		void LoadSave( IniInterface& conf );
 		
 		bool operator ==( const DebugOptions& right ) const
 		{
-			return OpEqu( bitset ) && OpEqu( FontWidth ) && OpEqu( FontHeight );
+			return OpEqu( bitset ) && OpEqu( FontWidth ) && OpEqu( FontHeight )
+				&& OpEqu( WindowWidth ) && OpEqu( WindowHeight );
 		}
 
 		bool operator !=( const DebugOptions& right ) const
@@ -441,6 +445,7 @@ struct Pcsx2Config
 			BackupSavestate		:1,
 		// enables simulated ejection of memory cards when loading savestates
 			McdEnableEjection	:1,
+			McdFolderAutoManage	:1,
 
 			MultitapPort0_Enabled:1,
 			MultitapPort1_Enabled:1,
