@@ -650,9 +650,9 @@ void P_COP2_Unknown( std::string& output )
 
 void label_decode( std::string& output, u32 addr )
 {
-	std::string buf;
-	ssprintf(buf, "->$0x%08X", addr);
-	output += buf;
+	char buffer[32];
+	sprintf(buffer, "->$0x%08X", addr);
+	output += std::string(buffer);
 }
 
 void jump_decode( std::string& output )
@@ -783,7 +783,7 @@ void BNE( std::string& output )
 	else if (disSimplify && rs != 0 && rt == 0)
 		disBranch(output, "bnez", rs);
 	else
-		disBranch(output, "bne", rt, rs);
+		disBranch(output, "bne", rs, rt);
 }
 
 void BLEZ( std::string& output )   { disBranch(output, "blez", DECODE_RS); }
@@ -847,7 +847,7 @@ void BNEL( std::string& output )
 	else if (disSimplify && rs != 0 && rt == 0)
 		disBranch(output, "bnezl", rs);
 	else
-		disBranch(output, "bnel", rt, rs);
+		disBranch(output, "bnel", rs, rt);
 }
 
 void BLEZL( std::string& output )  { disBranch(output, "blezl", DECODE_RS); }
@@ -869,6 +869,9 @@ void disMemAccess( std::string& output, const char* name, int cop = 0)
 	case 2:
 		rt = COP2_REG_FP[DECODE_FT];
 		break;
+	default:
+		rt = "???";
+		break;
 	}
 
 	const char* rs = GPR_REG[DECODE_RS];
