@@ -90,7 +90,10 @@ void SysMtgsThread::OnStart()
 
 SysMtgsThread::~SysMtgsThread() throw()
 {
-	_parent::Cancel();
+	try {
+		_parent::Cancel();
+	}
+	DESTRUCTOR_CATCHALL
 }
 
 void SysMtgsThread::OnResumeReady()
@@ -179,6 +182,7 @@ static void dummyIrqCallback()
 
 void SysMtgsThread::OpenPlugin()
 {
+
 	if( m_PluginOpened ) return;
 
 	memcpy( RingBuffer.Regs, PS2MEM_GS, sizeof(PS2MEM_GS) );
@@ -192,24 +196,8 @@ void SysMtgsThread::OpenPlugin()
 	else
 		result = GSopen( (void*)pDsp, "PCSX2", renderswitch ? 2 : 1 );
 
-	// Vsync on / off ?
-	if( renderswitch )
-	{
-		Console.Indent(2).WriteLn( "Forced software switch enabled." );
-		if ( EmuConfig.GS.VsyncEnable )
-		{
-			// Better turn Vsync off now, as in most cases sw rendering is not fast enough to support a steady 60fps.
-			// Having Vsync still enabled then means a big cut in speed and sloppy rendering.
-			// It's possible though that some users have very fast machines, and rather kept Vsync enabled,
-			// but let's assume this is the minority. At least for now ;)
-			GSsetVsync( false );
-			Console.Indent(2).WriteLn( "Vsync temporarily disabled" );
-		}
-	}
-	else
-	{
-		GSsetVsync( EmuConfig.GS.FrameLimitEnable && EmuConfig.GS.VsyncEnable );
-	}
+
+	GSsetVsync(EmuConfig.GS.FrameLimitEnable && EmuConfig.GS.VsyncEnable);
 
 	if( result != 0 )
 	{
