@@ -66,6 +66,13 @@ u64 SysPluginBindings::McdGetCRC( uint port, uint slot )
 	return Mcd->McdGetCRC( (PS2E_THISPTR) Mcd, port, slot );
 }
 
+void SysPluginBindings::McdNextFrame( uint port, uint slot ) {
+	Mcd->McdNextFrame( (PS2E_THISPTR) Mcd, port, slot );
+}
+
+void SysPluginBindings::McdReIndex( uint port, uint slot, const wxString& filter ) {
+	Mcd->McdReIndex( (PS2E_THISPTR) Mcd, port, slot, filter );
+}
 
 // ----------------------------------------------------------------------------
 // Yay, order of this array shouldn't be important. :)
@@ -320,6 +327,7 @@ _DEV9writeDMA8Mem  DEV9writeDMA8Mem;
 #endif
 _DEV9irqCallback   DEV9irqCallback;
 _DEV9irqHandler    DEV9irqHandler;
+_DEV9async         DEV9async;
 
 // USB
 _USBopen           USBopen;
@@ -630,6 +638,7 @@ static const LegacyApi_ReqMethod s_MethMessReq_DEV9[] =
 
 static const LegacyApi_OptMethod s_MethMessOpt_DEV9[] =
 {
+	{ "DEV9async", (vMeth**)&DEV9async },
 	{ NULL }
 };
 
@@ -853,19 +862,22 @@ SysCorePlugins::PluginStatus_t::PluginStatus_t( PluginsEnum_t _pid, const wxStri
 
 	_PS2EgetLibName		GetLibName		= (_PS2EgetLibName)		Lib.GetSymbol( L"PS2EgetLibName" );
 	_PS2EgetLibVersion2	GetLibVersion2	= (_PS2EgetLibVersion2)	Lib.GetSymbol( L"PS2EgetLibVersion2" );
-#ifdef __linux__
-	_PS2EsetEmuVersion	SetEmuVersion	= NULL;
-#else
-	_PS2EsetEmuVersion	SetEmuVersion	= (_PS2EsetEmuVersion)	Lib.GetSymbol( L"PS2EsetEmuVersion" );
-#endif
 
 	if( GetLibName == NULL || GetLibVersion2 == NULL )
 		throw Exception::PluginLoadError( pid ).SetStreamName(Filename)
 			.SetDiagMsg(L"%s plugin init failed: Method binding failure on GetLibName or GetLibVersion2.")
 			.SetUserMsg(_( "The configured %s plugin is not a PCSX2 plugin, or is for an older unsupported version of PCSX2."));
 
-	if( SetEmuVersion != NULL )
-		SetEmuVersion( "PCSX2", (0ul << 24) | (9ul<<16) | (7ul<<8) | 0 );
+	// Only Windows GSdx uses this. Should be removed in future after GSdx no longer relies on it to show the new config dialog.
+#ifdef _WIN32
+	// Since only Windows Gsdx has this symbol, that means every other plugin is going to cause error messages to be logged.
+	// Let's not do that for a hack function.
+	if (Lib.HasSymbol(L"PS2EsetEmuVersion")) {
+		_PS2EsetEmuVersion	SetEmuVersion = (_PS2EsetEmuVersion)Lib.GetSymbol(L"PS2EsetEmuVersion");
+		if (SetEmuVersion)
+			SetEmuVersion("PCSX2", (PCSX2_VersionHi << 24) | (PCSX2_VersionMid << 16) | (PCSX2_VersionLo << 8) | 0);
+	}
+#endif
 
 	Name = fromUTF8( GetLibName() );
 	int version = GetLibVersion2( tbl_PluginInfo[pid].typemask );
