@@ -18,6 +18,7 @@
 #include "IopCommon.h"
 #include "VUmicro.h"
 #include "newVif.h"
+#include "MTVU.h"
 
 #include "SamplProf.h"
 
@@ -72,9 +73,23 @@ void* RecompiledCodeReserve::Reserve( size_t size, uptr base, uptr upper_bounds
 {
 	if (!_parent::Reserve(size, base, upper_bounds)) return NULL;
 	_registerProfiler();
+
+	// Pre-Allocate the first block (to reduce the number of segmentation fault
+	// in debugger)
+	DoCommitAndProtect(0);
+
 	return m_baseptr;
 }
 
+void RecompiledCodeReserve::Reset()
+{
+	_parent::Reset();
+
+	// Pre-Allocate the first block (to reduce the number of segmentation fault
+	// in debugger)
+	DoCommitAndProtect(0);
+}
+
 
 // Sets the abbreviated name used by the profiler.  Name should be under 10 characters long.
 // After a name has been set, a profiler source will be automatically registered and cleared
@@ -99,7 +114,7 @@ void RecompiledCodeReserve::OnCommittedBlock( void* block )
 		// the assembly dump more cleanly.  We don't clear the block on Release builds since
 		// it can add a noticeable amount of overhead to large block recompilations.
 
-		memset_sse_a<0xcc>( block, m_blocksize * __pagesize );
+		memset(block, 0xCC, m_blocksize * __pagesize);
 	}
 }
 
@@ -195,7 +210,7 @@ void SysLogMachineCaps()
 	if ( !PCSX2_isReleaseVersion )
 	{
 		Console.WriteLn(Color_StrongGreen, "PCSX2 %u.%u.%u-%lld %s"
-#ifndef openSUSE
+#ifndef DISABLE_BUILD_DATE
 			"- compiled on " __DATE__
 #endif
 			, PCSX2_VersionHi, PCSX2_VersionMid, PCSX2_VersionLo,
@@ -204,7 +219,7 @@ void SysLogMachineCaps()
 	}
 	else { // shorter release version string
 		Console.WriteLn(Color_StrongGreen, "PCSX2 %u.%u.%u-%lld"
-#ifndef openSUSE
+#ifndef DISABLE_BUILD_DATE
 			"- compiled on " __DATE__
 #endif
 			, PCSX2_VersionHi, PCSX2_VersionMid, PCSX2_VersionLo,
@@ -252,6 +267,7 @@ void SysLogMachineCaps()
 	if( x86caps.hasStreamingSIMD4Extensions )		features[0].Add( L"SSE4.1" );
 	if( x86caps.hasStreamingSIMD4Extensions2 )		features[0].Add( L"SSE4.2" );
 	if( x86caps.hasAVX )							features[0].Add( L"AVX" );
+	if( x86caps.hasAVX2 )							features[0].Add( L"AVX2" );
 	if( x86caps.hasFMA)								features[0].Add( L"FMA" );
 
 	if( x86caps.hasMultimediaExtensionsExt )		features[1].Add( L"MMX2  " );
@@ -321,8 +337,11 @@ CpuInitializer< CpuType >::CpuInitializer()
 template< typename CpuType >
 CpuInitializer< CpuType >::~CpuInitializer() throw()
 {
-	if (MyCpu)
-		MyCpu->Shutdown();
+	try {
+		if (MyCpu)
+			MyCpu->Shutdown();
+	}
+	DESTRUCTOR_CATCHALL
 }
 
 // --------------------------------------------------------------------------------------
@@ -365,7 +384,10 @@ SysMainMemory::SysMainMemory()
 
 SysMainMemory::~SysMainMemory() throw()
 {
-	ReleaseAll();
+	try {
+		ReleaseAll();
+	}
+	DESTRUCTOR_CATCHALL
 }
 
 void SysMainMemory::ReserveAll()
@@ -418,6 +440,12 @@ void SysMainMemory::DecommitAll()
 	Console.WriteLn( Color_Blue, "Decommitting host memory for virtual systems..." );
 	ConsoleIndentScope indent(1);
 
+	// On linux, the MTVU isn't empty and the thread still uses the m_ee/m_vu memory
+	vu1Thread.WaitVU();
+	// The EE thread must be stopped here command mustn't be send
+	// to the ring. Let's call it an extra safety valve :)
+	vu1Thread.Reset();
+
 	m_ee.Decommit();
 	m_iop.Decommit();
 	m_vu.Decommit();
