@@ -31,6 +31,7 @@
 #include "Elfheader.h"
 #include "Patch.h"
 #include "R5900Exceptions.h"
+#include "Sio.h"
 
 __aligned16 SysMtgsThread mtgsThread;
 __aligned16 AppCoreThread CoreThread;
@@ -88,7 +89,10 @@ AppCoreThread::AppCoreThread() : SysCoreThread()
 
 AppCoreThread::~AppCoreThread() throw()
 {
-	_parent::Cancel();		// use parent's, skips thread affinity check.
+	try {
+		_parent::Cancel();		// use parent's, skips thread affinity check.
+	}
+	DESTRUCTOR_CATCHALL
 }
 
 static void _Cancel()
@@ -213,7 +217,7 @@ void AppCoreThread::OnPause()
 // Load Game Settings found in database
 // (game fixes, round modes, clamp modes, etc...)
 // Returns number of gamefixes set
-static int loadGameSettings(Pcsx2Config& dest, const Game_Data& game, bool verbose = true) {
+static int loadGameSettings(Pcsx2Config& dest, const Game_Data& game) {
 	if( !game.IsOk() ) return 0;
 
 	int  gf  = 0;
@@ -223,7 +227,7 @@ static int loadGameSettings(Pcsx2Config& dest, const Game_Data& game, bool verbo
 		SSE_RoundMode eeRM = (SSE_RoundMode)game.getInt("eeRoundMode");
 		if (EnumIsValid(eeRM))
 		{
-			if(verbose) Console.WriteLn("(GameDB) Changing EE/FPU roundmode to %d [%s]", eeRM, EnumToString(eeRM));
+			PatchesCon->WriteLn("(GameDB) Changing EE/FPU roundmode to %d [%s]", eeRM, EnumToString(eeRM));
 			dest.Cpu.sseMXCSR.SetRoundMode(eeRM);
 			++gf;
 		}
@@ -234,7 +238,7 @@ static int loadGameSettings(Pcsx2Config& dest, const Game_Data& game, bool verbo
 		SSE_RoundMode vuRM = (SSE_RoundMode)game.getInt("vuRoundMode");
 		if (EnumIsValid(vuRM))
 		{
-			if(verbose) Console.WriteLn("(GameDB) Changing VU0/VU1 roundmode to %d [%s]", vuRM, EnumToString(vuRM));
+			PatchesCon->WriteLn("(GameDB) Changing VU0/VU1 roundmode to %d [%s]", vuRM, EnumToString(vuRM));
 			dest.Cpu.sseVUMXCSR.SetRoundMode(vuRM);
 			++gf;
 		}
@@ -242,7 +246,7 @@ static int loadGameSettings(Pcsx2Config& dest, const Game_Data& game, bool verbo
 
 	if (game.keyExists("eeClampMode")) {
 		int clampMode = game.getInt("eeClampMode");
-		if(verbose) Console.WriteLn("(GameDB) Changing EE/FPU clamp mode [mode=%d]", clampMode);
+		PatchesCon->WriteLn("(GameDB) Changing EE/FPU clamp mode [mode=%d]", clampMode);
 		dest.Cpu.Recompiler.fpuOverflow			= (clampMode >= 1);
 		dest.Cpu.Recompiler.fpuExtraOverflow	= (clampMode >= 2);
 		dest.Cpu.Recompiler.fpuFullMode			= (clampMode >= 3);
@@ -251,7 +255,7 @@ static int loadGameSettings(Pcsx2Config& dest, const Game_Data& game, bool verbo
 
 	if (game.keyExists("vuClampMode")) {
 		int clampMode = game.getInt("vuClampMode");
-		if(verbose) Console.WriteLn("(GameDB) Changing VU0/VU1 clamp mode [mode=%d]", clampMode);
+		PatchesCon->WriteLn("(GameDB) Changing VU0/VU1 clamp mode [mode=%d]", clampMode);
 		dest.Cpu.Recompiler.vuOverflow			= (clampMode >= 1);
 		dest.Cpu.Recompiler.vuExtraOverflow		= (clampMode >= 2);
 		dest.Cpu.Recompiler.vuSignOverflow		= (clampMode >= 3);
@@ -261,7 +265,7 @@ static int loadGameSettings(Pcsx2Config& dest, const Game_Data& game, bool verbo
 
 	if (game.keyExists("mvuFlagSpeedHack")) {
 		bool vuFlagHack = game.getInt("mvuFlagSpeedHack") ? 1 : 0;
-		if(verbose) Console.WriteLn("(GameDB) Changing mVU flag speed hack [mode=%d]", vuFlagHack);
+		PatchesCon->WriteLn("(GameDB) Changing mVU flag speed hack [mode=%d]", vuFlagHack);
 		dest.Speedhacks.vuFlagHack = vuFlagHack;
 		gf++;
 	}
@@ -275,7 +279,7 @@ static int loadGameSettings(Pcsx2Config& dest, const Game_Data& game, bool verbo
 		{
 			bool enableIt = game.getBool(key);
 			dest.Gamefixes.Set(id, enableIt);
-			if(verbose) Console.WriteLn(L"(GameDB) %s Gamefix: " + key, enableIt ? L"Enabled" : L"Disabled" );
+			PatchesCon->WriteLn(L"(GameDB) %s Gamefix: " + key, enableIt ? L"Enabled" : L"Disabled");
 			gf++;
 
 			// The LUT is only used for 1 game so we allocate it only when the gamefix is enabled (save 4MB)
@@ -293,6 +297,24 @@ static int loadGameSettings(Pcsx2Config& dest, const Game_Data& game, bool verbo
 // File scope since it gets reset externally when rebooting
 static wxString curGameKey;
 
+// PatchesCon points to either Console or ConsoleWriter_Null, such that if we're in Devel mode
+// or the user enabled the devel/verbose console it prints all patching info whenever it's applied,
+// else it prints the patching info only once - right after boot.
+const IConsoleWriter *PatchesCon = &Console;
+
+static void SetupPatchesCon(bool verbose)
+{
+	bool devel = false;
+#ifdef PCSX2_DEVBUILD
+	devel = true;
+#endif
+
+	if (verbose || DevConWriterEnabled || devel)
+		PatchesCon = &Console;
+	else
+		PatchesCon = &ConsoleWriter_Null;
+}
+
 void AppCoreThread::ApplySettings( const Pcsx2Config& src )
 {
 	// 'fixup' is the EmuConfig we're going to upload to the emulator, which very well may
@@ -326,6 +348,7 @@ void AppCoreThread::ApplySettings( const Pcsx2Config& src )
 
 	wxString gameName;
 	wxString gameCompat;
+	wxString gameMemCardFilter;
 
 	int numberLoadedCheats;
 	int numberLoadedWideScreenPatches;
@@ -336,6 +359,8 @@ void AppCoreThread::ApplySettings( const Pcsx2Config& src )
 
 	const wxString newGameKey( SysGetDiscID() );
 	const bool verbose( newGameKey != curGameKey );
+	SetupPatchesCon(verbose);
+
 	curGameKey = newGameKey;
 
 	if (!curGameKey.IsEmpty())
@@ -348,20 +373,27 @@ void AppCoreThread::ApplySettings( const Pcsx2Config& src )
 				gameName   = game.getString("Name");
 				gameName  += L" (" + game.getString("Region") + L")";
 				gameCompat = L" [Status = "+compatToStringWX(compat)+L"]";
+				gameMemCardFilter = game.getString("MemCardFilter");
 			}
 
 			if (EmuConfig.EnablePatches) {
 				if (int patches = InitPatches(gameCRC, game)) {
 					gamePatch.Printf(L" [%d Patches]", patches);
-					if (verbose) Console.WriteLn(Color_Green, "(GameDB) Patches Loaded: %d", patches);
+					PatchesCon->WriteLn(Color_Green, "(GameDB) Patches Loaded: %d", patches);
 				}
-				if (int fixes = loadGameSettings(fixup, game, verbose)) {
+				if (int fixes = loadGameSettings(fixup, game)) {
 					gameFixes.Printf(L" [%d Fixes]", fixes);
 				}
 			}
 		}
 	}
 
+	if (!gameMemCardFilter.IsEmpty()) {
+		sioSetGameSerial(gameMemCardFilter);
+	} else {
+		sioSetGameSerial(curGameKey);
+	}
+
 	if (gameName.IsEmpty() && gameSerial.IsEmpty() && gameCRC.IsEmpty())
 	{
 		// if all these conditions are met, it should mean that we're currently running BIOS code.
@@ -398,8 +430,7 @@ void AppCoreThread::ApplySettings( const Pcsx2Config& src )
 			wxString cheats_ws_archive = Path::Combine(PathDefs::GetProgramDataDir(), wxFileName(L"cheats_ws.zip"));
 
 			if (numberDbfCheatsLoaded = LoadCheatsFromZip(gameCRC, cheats_ws_archive)) {
-				if (verbose || DevConWriterEnabled)
-					Console.WriteLn(Color_Green, "(Wide Screen Cheats DB) Patches Loaded: %d", numberDbfCheatsLoaded);
+				PatchesCon->WriteLn(Color_Green, "(Wide Screen Cheats DB) Patches Loaded: %d", numberDbfCheatsLoaded);
 				gameWsHacks.Printf(L" [%d widescreen hacks]", numberDbfCheatsLoaded);
 			}
 		}
@@ -494,6 +525,7 @@ void AppCoreThread::GameStartingInThread()
 	m_ExecMode = ExecMode_Paused;
 	OnResumeReady();
 	_reset_stuff_as_needed();
+	ClearMcdEjectTimeoutNow(); // probably safe to do this when a game boots, eliminates annoying prompts
 	m_ExecMode = ExecMode_Opened;
 	
 	_parent::GameStartingInThread();
@@ -705,8 +737,11 @@ ScopedCoreThreadClose::ScopedCoreThreadClose()
 ScopedCoreThreadClose::~ScopedCoreThreadClose() throw()
 {
 	if( m_alreadyScoped ) return;
-	_parent::DoResume();
-	ScopedCore_IsFullyClosed = false;
+	try {
+		_parent::DoResume();
+		ScopedCore_IsFullyClosed = false;
+	}
+	DESTRUCTOR_CATCHALL
 }
 
 ScopedCoreThreadPause::ScopedCoreThreadPause( BaseSysExecEvent_ScopedCore* abuse_me )
@@ -732,8 +767,11 @@ ScopedCoreThreadPause::ScopedCoreThreadPause( BaseSysExecEvent_ScopedCore* abuse
 ScopedCoreThreadPause::~ScopedCoreThreadPause() throw()
 {
 	if( m_alreadyScoped ) return;
-	_parent::DoResume();
-	ScopedCore_IsPaused = false;
+	try {
+		_parent::DoResume();
+		ScopedCore_IsPaused = false;
+	}
+	DESTRUCTOR_CATCHALL
 }
 
 ScopedCoreThreadPopup::ScopedCoreThreadPopup()
