@@ -16,8 +16,8 @@
 #include "PrecompiledHeader.h"
 #include "Utilities/SafeArray.inl"
 #include <wx/file.h>
-
-#include "MemoryCardFile.h"
+#include <wx/dir.h>
+#include <wx/stopwatch.h>
 
 // IMPORTANT!  If this gets a macro redefinition error it means PluginCallbacks.h is included
 // in a global-scope header, and that's a BAD THING.  Include it only into modules that need
@@ -26,12 +26,16 @@
 struct Component_FileMcd;
 #define PS2E_THISPTR Component_FileMcd*
 
+#include "MemoryCardFile.h"
+#include "MemoryCardFolder.h"
+
 #include "System.h"
 #include "AppConfig.h"
 
 #include "svnrev.h"
 
 #include <wx/ffile.h>
+#include <map>
 
 static const int MCD_SIZE	= 1024 *  8  * 16;		// Legacy PSX card default size
 
@@ -70,7 +74,7 @@ public:
 	s32  Save		( uint slot, const u8 *src, u32 adr, int size );
 	s32  EraseBlock	( uint slot, u32 adr );
 	u64  GetCRC		( uint slot );
-
+	
 protected:
 	bool Seek( wxFFile& f, u32 adr );
 	bool Create( const wxString& mcdFile, uint sizeInMB );
@@ -167,7 +171,12 @@ void FileMemoryCard::Open()
 			cont = true;
 		}
 
-		Console.WriteLn( cont ? Color_Gray : Color_Green, L"McdSlot %u: " + str, slot );
+		if ( g_Conf->Mcd[slot].Type != MemoryCardType::MemoryCard_File ) {
+			str = L"[is not memcard file]";
+			cont = true;
+		}
+
+		Console.WriteLn( cont ? Color_Gray : Color_Green, L"McdSlot %u [File]: " + str, slot );
 		if( cont ) continue;
 
 		const wxULongLong fsz = fname.GetSize();
@@ -403,8 +412,9 @@ u64 FileMemoryCard::GetCRC( uint slot )
 
 struct Component_FileMcd
 {
-	PS2E_ComponentAPI_Mcd	api;	// callbacks the plugin provides back to the emulator
-	FileMemoryCard			impl;	// class-based implementations we refer to when API is invoked
+	PS2E_ComponentAPI_Mcd      api;  // callbacks the plugin provides back to the emulator
+	FileMemoryCard             impl; // class-based implementations we refer to when API is invoked
+	FolderMemoryCardAggregator implFolder;
 
 	Component_FileMcd();
 };
@@ -418,47 +428,152 @@ uint FileMcd_ConvertToSlot( uint port, uint slot )
 
 static void PS2E_CALLBACK FileMcd_EmuOpen( PS2E_THISPTR thisptr, const PS2E_SessionInfo *session )
 {
+	// detect inserted memory card types
+	for ( uint slot = 0; slot < 8; ++slot ) {
+		if ( g_Conf->Mcd[slot].Enabled ) {
+			MemoryCardType type = MemoryCardType::MemoryCard_File; // default to file if we can't find anything at the path so it gets auto-generated
+
+			const wxString path = g_Conf->FullpathToMcd( slot );
+			if ( wxFileExists( path ) ) {
+				type = MemoryCardType::MemoryCard_File;
+			} else if ( wxDirExists( path ) ) {
+				type = MemoryCardType::MemoryCard_Folder;
+			}
+
+			g_Conf->Mcd[slot].Type = type;
+		}
+	}
+
 	thisptr->impl.Open();
+	thisptr->implFolder.SetFiltering( g_Conf->EmuOptions.McdFolderAutoManage );
+	thisptr->implFolder.Open();
 }
 
 static void PS2E_CALLBACK FileMcd_EmuClose( PS2E_THISPTR thisptr )
 {
+	thisptr->implFolder.Close();
 	thisptr->impl.Close();
 }
 
 static s32 PS2E_CALLBACK FileMcd_IsPresent( PS2E_THISPTR thisptr, uint port, uint slot )
 {
-	return thisptr->impl.IsPresent( FileMcd_ConvertToSlot( port, slot ) );
+	const uint combinedSlot = FileMcd_ConvertToSlot( port, slot );
+	switch ( g_Conf->Mcd[combinedSlot].Type ) {
+	case MemoryCardType::MemoryCard_File:
+		return thisptr->impl.IsPresent( combinedSlot );
+	case MemoryCardType::MemoryCard_Folder:
+		return thisptr->implFolder.IsPresent( combinedSlot );
+	default:
+		return false;
+	}
 }
 
 static void PS2E_CALLBACK FileMcd_GetSizeInfo( PS2E_THISPTR thisptr, uint port, uint slot, PS2E_McdSizeInfo* outways )
 {
-	thisptr->impl.GetSizeInfo( FileMcd_ConvertToSlot( port, slot ), *outways );
+	const uint combinedSlot = FileMcd_ConvertToSlot( port, slot );
+	switch ( g_Conf->Mcd[combinedSlot].Type ) {
+	case MemoryCardType::MemoryCard_File:
+		thisptr->impl.GetSizeInfo( combinedSlot, *outways );
+		break;
+	case MemoryCardType::MemoryCard_Folder:
+		thisptr->implFolder.GetSizeInfo( combinedSlot, *outways );
+		break;
+	default:
+		return;
+	}
 }
 
 static bool PS2E_CALLBACK FileMcd_IsPSX( PS2E_THISPTR thisptr, uint port, uint slot )
 {
-	return thisptr->impl.IsPSX( FileMcd_ConvertToSlot( port, slot ) );
+	const uint combinedSlot = FileMcd_ConvertToSlot( port, slot );
+	switch ( g_Conf->Mcd[combinedSlot].Type ) {
+	case MemoryCardType::MemoryCard_File:
+		return thisptr->impl.IsPSX( combinedSlot );
+	case MemoryCardType::MemoryCard_Folder:
+		return thisptr->implFolder.IsPSX( combinedSlot );
+	default:
+		return false;
+	}
 }
 
 static s32 PS2E_CALLBACK FileMcd_Read( PS2E_THISPTR thisptr, uint port, uint slot, u8 *dest, u32 adr, int size )
 {
-	return thisptr->impl.Read( FileMcd_ConvertToSlot( port, slot ), dest, adr, size );
+	const uint combinedSlot = FileMcd_ConvertToSlot( port, slot );
+	switch ( g_Conf->Mcd[combinedSlot].Type ) {
+	case MemoryCardType::MemoryCard_File:
+		return thisptr->impl.Read( combinedSlot, dest, adr, size );
+	case MemoryCardType::MemoryCard_Folder:
+		return thisptr->implFolder.Read( combinedSlot, dest, adr, size );
+	default:
+		return 0;
+	}
 }
 
 static s32 PS2E_CALLBACK FileMcd_Save( PS2E_THISPTR thisptr, uint port, uint slot, const u8 *src, u32 adr, int size )
 {
-	return thisptr->impl.Save( FileMcd_ConvertToSlot( port, slot ), src, adr, size );
+	const uint combinedSlot = FileMcd_ConvertToSlot( port, slot );
+	switch ( g_Conf->Mcd[combinedSlot].Type ) {
+	case MemoryCardType::MemoryCard_File:
+		return thisptr->impl.Save( combinedSlot, src, adr, size );
+	case MemoryCardType::MemoryCard_Folder:
+		return thisptr->implFolder.Save( combinedSlot, src, adr, size );
+	default:
+		return 0;
+	}
 }
 
 static s32 PS2E_CALLBACK FileMcd_EraseBlock( PS2E_THISPTR thisptr, uint port, uint slot, u32 adr )
 {
-	return thisptr->impl.EraseBlock( FileMcd_ConvertToSlot( port, slot ), adr );
+	const uint combinedSlot = FileMcd_ConvertToSlot( port, slot );
+	switch ( g_Conf->Mcd[combinedSlot].Type ) {
+	case MemoryCardType::MemoryCard_File:
+		return thisptr->impl.EraseBlock( combinedSlot, adr );
+	case MemoryCardType::MemoryCard_Folder:
+		return thisptr->implFolder.EraseBlock( combinedSlot, adr );
+	default:
+		return 0;
+	}
 }
 
 static u64 PS2E_CALLBACK FileMcd_GetCRC( PS2E_THISPTR thisptr, uint port, uint slot )
 {
-	return thisptr->impl.GetCRC( FileMcd_ConvertToSlot( port, slot ) );
+	const uint combinedSlot = FileMcd_ConvertToSlot( port, slot );
+	switch ( g_Conf->Mcd[combinedSlot].Type ) {
+	case MemoryCardType::MemoryCard_File:
+		return thisptr->impl.GetCRC( combinedSlot );
+	case MemoryCardType::MemoryCard_Folder:
+		return thisptr->implFolder.GetCRC( combinedSlot );
+	default:
+		return 0;
+	}
+}
+
+static void PS2E_CALLBACK FileMcd_NextFrame( PS2E_THISPTR thisptr, uint port, uint slot ) {
+	const uint combinedSlot = FileMcd_ConvertToSlot( port, slot );
+	switch ( g_Conf->Mcd[combinedSlot].Type ) {
+	//case MemoryCardType::MemoryCard_File:
+	//	thisptr->impl.NextFrame( combinedSlot );
+	//	break;
+	case MemoryCardType::MemoryCard_Folder:
+		thisptr->implFolder.NextFrame( combinedSlot );
+		break;
+	default:
+		return;
+	}
+}
+
+static void PS2E_CALLBACK FileMcd_ReIndex( PS2E_THISPTR thisptr, uint port, uint slot, const wxString& filter ) {
+	const uint combinedSlot = FileMcd_ConvertToSlot( port, slot );
+	switch ( g_Conf->Mcd[combinedSlot].Type ) {
+	//case MemoryCardType::MemoryCard_File:
+	//	thisptr->impl.ReIndex( combinedSlot, filter );
+	//	break;
+	case MemoryCardType::MemoryCard_Folder:
+		thisptr->implFolder.ReIndex( combinedSlot, g_Conf->EmuOptions.McdFolderAutoManage, filter );
+		break;
+	default:
+		return;
+	}
 }
 
 Component_FileMcd::Component_FileMcd()
@@ -475,6 +590,8 @@ Component_FileMcd::Component_FileMcd()
 	api.McdSave			= FileMcd_Save;
 	api.McdEraseBlock	= FileMcd_EraseBlock;
 	api.McdGetCRC		= FileMcd_GetCRC;
+	api.McdNextFrame    = FileMcd_NextFrame;
+	api.McdReIndex      = FileMcd_ReIndex;
 }
 
 
@@ -547,32 +664,6 @@ extern "C" const PS2E_LibraryAPI* FileMcd_InitAPI( const PS2E_EmulatorInfo* emui
 	return &FileMcd_Library;
 }
 
-// --------------------------------------------------------------------------------------
-//  Currently Unused Superblock Header Struct
-// --------------------------------------------------------------------------------------
-// (provided for reference purposes)
-
-struct superblock
-{
-	char magic[28]; 			// 0x00
-	char version[12]; 			// 0x1c
-	u16 page_len; 				// 0x28
-	u16 pages_per_cluster;	 	// 0x2a
-	u16 pages_per_block;		// 0x2c
-	u16 unused; 				// 0x2e
-	u32 clusters_per_card;	 	// 0x30
-	u32 alloc_offset; 			// 0x34
-	u32 alloc_end; 				// 0x38
-	u32 rootdir_cluster;		// 0x3c
-	u32 backup_block1;			// 0x40
-	u32 backup_block2;			// 0x44
-	u32 ifc_list[32]; 			// 0x50
-	u32 bad_block_list[32]; 	// 0xd0
-	u8 card_type; 				// 0x150
-	u8 card_flags; 				// 0x151
-};
-
-
 //Tests if a string is a valid name for a new file within a specified directory.
 //returns true if:
 //     - the file name has a minimum length of minNumCharacters chars (default is 5 chars: at least 1 char + '.' + 3-chars extension)
@@ -597,6 +688,11 @@ bool isValidNewFilename( wxString filenameStringToTest, wxDirName atBasePath, wx
 		out_errorMessage = _("File name already exists");
 		return false;
 	}
+	if ( wxDirExists( (atBasePath + wxFileName(filenameStringToTest)).GetFullPath() ))
+	{
+		out_errorMessage = _( "File name already exists" );
+		return false;
+	}
 
 	wxFile fp;
 	if( !fp.Create( (atBasePath + wxFileName(filenameStringToTest)).GetFullPath() ))
