@@ -177,6 +177,7 @@ namespace Panels
 		pxRadioPanel*		m_panel_RecIOP;
 		pxCheckBox*			m_check_EECacheEnable;
 		AdvancedOptionsFPU*	m_advancedOptsFpu;
+		wxButton *m_button_RestoreDefaults;
 
 	public:
 		CpuPanelEE( wxWindow* parent );
@@ -196,6 +197,7 @@ namespace Panels
 		pxRadioPanel*				m_panel_VU0;
 		pxRadioPanel*				m_panel_VU1;
 		Panels::AdvancedOptionsVU*	m_advancedOptsVu;
+		wxButton *m_button_RestoreDefaults;
 
 	public:
 		CpuPanelVU( wxWindow* parent );
@@ -324,6 +326,8 @@ namespace Panels
 		pxCheckBox*		m_check_Enable;
 		wxButton*		m_button_Defaults;
 
+		wxPanelWithHelpers* m_eeSliderPanel;
+		wxPanelWithHelpers* m_vuSliderPanel;
 		wxSlider*		m_slider_eecycle;
 		wxSlider*		m_slider_vustealer;
 		pxStaticText*	m_msg_eecycle;
@@ -351,7 +355,6 @@ namespace Panels
 
 		void OnEnable_Toggled( wxCommandEvent& evt );
 		void Defaults_Click( wxCommandEvent& evt );
-		void Slider_Click(wxScrollEvent &event);
 		void EECycleRate_Scroll(wxScrollEvent &event);
 		void VUCycleRate_Scroll(wxScrollEvent &event);
 	};
@@ -560,7 +563,10 @@ namespace Panels
 		public:
 			virtual ~EnumThread() throw()
 			{
-				pxThread::Cancel();
+				try {
+					pxThread::Cancel();
+				}
+				DESTRUCTOR_CATCHALL
 			}
 
 			EnumThread( PluginSelectorPanel& master );
