@@ -227,7 +227,7 @@ protected:
 IMPLEMENT_DYNAMIC_CLASS(ApplyPluginsDialog, WaitForTaskDialog)
 
 ApplyPluginsDialog::ApplyPluginsDialog( BaseApplicableConfigPanel* panel )
-	: WaitForTaskDialog( _("Applying settings...") )
+: WaitForTaskDialog(_("Applying settings...")), m_panel(NULL)
 {
 	GetSysExecutorThread().PostEvent( new SysExecEvent_ApplyPlugins( this, m_sync ) );
 }
@@ -546,7 +546,7 @@ void Panels::PluginSelectorPanel::DoRefresh()
 	wxCommandEvent evt( pxEVT_ShowStatusBar );
 	GetEventHandler()->AddPendingEvent( evt );
 
-	m_EnumeratorThread.Delete() = new EnumThread( *this );
+	m_EnumeratorThread.Reassign(new EnumThread( *this ));
 
 	if( DisableThreading )
 		m_EnumeratorThread->DoNextPlugin( 0 );
@@ -642,20 +642,16 @@ void Panels::PluginSelectorPanel::OnConfigure_Clicked( wxCommandEvent& evt )
 		ScopedCoreThreadPause paused_core( new SysExecEvent_SaveSinglePlugin(pid) );
 		if (!CorePlugins.AreLoaded())
 		{
-			typedef void	(CALLBACK* SetDirFnptr)( const char* dir );
-
-			if( SetDirFnptr func = (SetDirFnptr)dynlib.GetSymbol( tbl_PluginInfo[pid].GetShortname() + L"setSettingsDir" ) )
-			{
-				func( GetSettingsFolder().ToString().mb_str(wxConvFile) );
-			}
-
-			if( SetDirFnptr func = (SetDirFnptr)dynlib.GetSymbol( tbl_PluginInfo[pid].GetShortname() + L"setLogDir" ) )
-			{
-				func( GetLogFolder().ToString().mb_str(wxConvFile) );
-			}
+			CorePlugins.Load(pid, filename);
+			CorePlugins.SendLogFolder();
+			CorePlugins.SendSettingsFolder();
+			configfunc();
+			CorePlugins.Unload(pid);
+		}
+		else
+		{
+			configfunc();
 		}
-
-		configfunc();
 	}
 }
 
