@@ -22,6 +22,8 @@
 #include "VU.h"
 #include "R3000A.h"
 
+using namespace x86Emitter;
+
 __tls_emit u8  *j8Ptr[32];
 __tls_emit u32 *j32Ptr[32];
 
@@ -133,9 +135,9 @@ int  _getFreeXMMreg()
 		_freeXMMreg(tempi);
 		return tempi;
 	}
-	Console.Error("*PCSX2*: XMM Reg Allocation Error in _getFreeXMMreg()!");
 
-	return -1;
+	pxFailDev("*PCSX2*: XMM Reg Allocation Error in _getFreeXMMreg()!");
+	throw Exception::FailedToAllocateRegister();
 }
 
 int _allocTempXMMreg(XMMSSEType type, int xmmreg) {
@@ -313,7 +315,8 @@ int _allocFPtoXMMreg(int xmmreg, int fpreg, int mode) {
 		return i;
 	}
 
-	if (xmmreg == -1) xmmreg = _getFreeXMMreg();
+	if (xmmreg == -1)
+		xmmreg = _getFreeXMMreg();
 
 	g_xmmtypes[xmmreg] = XMMT_FPS;
 	xmmregs[xmmreg].inuse = 1;
@@ -378,7 +381,8 @@ int _allocGPRtoXMMreg(int xmmreg, int gprreg, int mode)
 		g_cpuHasConstReg &= ~(1<<gprreg);
 	}
 
-	if (xmmreg == -1) xmmreg = _getFreeXMMreg();
+	if (xmmreg == -1)
+		xmmreg = _getFreeXMMreg();
 
 	g_xmmtypes[xmmreg] = XMMT_INT;
 	xmmregs[xmmreg].inuse = 1;
@@ -455,9 +459,8 @@ int _allocFPACCtoXMMreg(int xmmreg, int mode)
 		return i;
 	}
 
-	if (xmmreg == -1) {
+	if (xmmreg == -1)
 		xmmreg = _getFreeXMMreg();
-	}
 
 	g_xmmtypes[xmmreg] = XMMT_FPS;
 	xmmregs[xmmreg].inuse = 1;
