@@ -28,6 +28,8 @@
 #include "sVU_Debug.h"
 #include "sVU_zerorec.h"
 
+using namespace x86Emitter;
+
 #ifdef _WIN32
 #pragma warning(disable:4244)
 #pragma warning(disable:4761)
@@ -292,14 +294,16 @@ void _recvuFMACAdd(VURegs * VU, int reg, int xyzw) {
 		break;
 	}
 
-	if (i==8) Console.Error("*PCSX2*: error , out of fmacs");
+	if (i==8) {
+		Console.Error("*PCSX2*: error , out of fmacs");
 //	VUM_LOG("adding FMAC pipe[%d]; reg %d", i, reg);
-
-	VU->fmac[i].enable = 1;
-	VU->fmac[i].sCycle = vucycle;
-	VU->fmac[i].Cycle = 3;
-	VU->fmac[i].xyzw = xyzw;
-	VU->fmac[i].reg = reg;
+	} else {
+		VU->fmac[i].enable = 1;
+		VU->fmac[i].sCycle = vucycle;
+		VU->fmac[i].Cycle = 3;
+		VU->fmac[i].xyzw = xyzw;
+		VU->fmac[i].reg = reg;
+	}
 }
 
 void _recvuFDIVAdd(VURegs * VU, int cycles) {
@@ -325,12 +329,14 @@ void _recvuIALUAdd(VURegs * VU, int reg, int cycles) {
 		break;
 	}
 
-	if (i==8) Console.Error("*PCSX2*: error , out of ialus");
-
-	VU->ialu[i].enable = 1;
-	VU->ialu[i].sCycle = vucycle;
-	VU->ialu[i].Cycle = cycles;
-	VU->ialu[i].reg = reg;
+	if (i==8) {
+		Console.Error("*PCSX2*: error , out of ialus");
+	} else {
+		VU->ialu[i].enable = 1;
+		VU->ialu[i].sCycle = vucycle;
+		VU->ialu[i].Cycle = cycles;
+		VU->ialu[i].reg = reg;
+	}
 }
 
 void _recvuTestIALUStalls(VURegs * VU, _VURegsNum *VUregsn) {
@@ -464,19 +470,19 @@ void SuperVUAnalyzeOp(VURegs *VU, _vuopinfo *info, _VURegsNum* pCodeRegs)
 {
 	_VURegsNum* lregs;
 	_VURegsNum* uregs;
-	int *ptr;
+	int *code_ptr;
 
 	lregs = pCodeRegs;
 	uregs = pCodeRegs+1;
 
-	ptr = (int*)&VU->Micro[pc];
+	code_ptr = (int*)&VU->Micro[pc];
 	pc += 8;
 
-	if (ptr[1] & 0x40000000) { // EOP
-		branch |= 8;
+	if (code_ptr[1] & 0x40000000) { // EOP
+		g_branch |= 8;
 	}
 
-	VU->code = ptr[1];
+	VU->code = code_ptr[1];
 	if (VU == &VU1) VU1regs_UPPER_OPCODE[VU->code & 0x3f](uregs);
 	else VU0regs_UPPER_OPCODE[VU->code & 0x3f](uregs);
 
@@ -539,13 +545,13 @@ void SuperVUAnalyzeOp(VURegs *VU, _vuopinfo *info, _VURegsNum* pCodeRegs)
 	if (uregs->VIread & (1 << REG_P)) { info->p |= 2; assert( VU == &VU1 ); }
 
 	// check upper flags
-	if (ptr[1] & 0x80000000) { // I flag
+	if (code_ptr[1] & 0x80000000) { // I flag
 		info->cycle = vucycle;
 		memzero(*lregs);
 	}
 	else {
 
-		VU->code = ptr[0];
+		VU->code = code_ptr[0];
 		if (VU == &VU1) VU1regs_LOWER_OPCODE[VU->code >> 25](lregs);
 		else VU0regs_LOWER_OPCODE[VU->code >> 25](lregs);
 
@@ -553,7 +559,7 @@ void SuperVUAnalyzeOp(VURegs *VU, _vuopinfo *info, _VURegsNum* pCodeRegs)
 		info->cycle = vucycle;
 
 		if (lregs->pipe == VUPIPE_BRANCH) {
-			branch |= 1;
+			g_branch |= 1;
 		}
 
 		if (lregs->VIwrite & (1 << REG_Q)) {
