@@ -62,14 +62,13 @@ extern bool RunLinuxDialog();
 static GSRenderer* s_gs = NULL;
 static void (*s_irq)() = NULL;
 static uint8* s_basemem = NULL;
-static int s_renderer = -1;
+static GSRendererType s_renderer = GSRendererType::Undefined;
 static bool s_framelimit = true;
 static bool s_vsync = false;
 static bool s_exclusive = true;
-#ifdef _WINDOWS
-static bool s_isgsopen2 = false; // boolean to remove some stuff from the config panel in new PCSX2's/
-#endif
-bool gsopen_done = false; // crash guard for GSgetTitleInfo2
+static const char *s_renderer_name = "";
+static const char *s_renderer_type = "";
+bool gsopen_done = false; // crash guard for GSgetTitleInfo2 and GSKeyEvent (replace with lock?)
 
 EXPORT_C_(uint32) PS2EgetLibType()
 {
@@ -83,21 +82,12 @@ EXPORT_C_(const char*) PS2EgetLibName()
 
 EXPORT_C_(uint32) PS2EgetLibVersion2(uint32 type)
 {
-	const uint32 revision = 0;
-	const uint32 build = 1;
+	const uint32 revision = 1;
+	const uint32 build = 0;
 
 	return (build << 0) | (revision << 8) | (PS2E_GS_VERSION << 16) | (PLUGIN_VERSION << 24);
 }
 
-#ifdef _WINDOWS
-
-EXPORT_C_(void) PS2EsetEmuVersion(const char* emuId, uint32 version)
-{
-	s_isgsopen2 = true;
-}
-
-#endif
-
 EXPORT_C_(uint32) PS2EgetCpuPlatform()
 {
 #ifdef _M_AMD64
@@ -155,7 +145,7 @@ EXPORT_C GSshutdown()
 
 	s_gs = NULL;
 
-	s_renderer = -1;
+	s_renderer = GSRendererType::Undefined;
 
 #ifdef _WINDOWS
 
@@ -189,25 +179,25 @@ EXPORT_C GSclose()
 	}
 }
 
-static int _GSopen(void** dsp, char* title, int renderer, int threads = -1)
+static int _GSopen(void** dsp, const char* title, GSRendererType renderer, int threads = -1)
 {
 	GSDevice* dev = NULL;
 
-	if(renderer == -1)
+	if(renderer == GSRendererType::Undefined)
 	{
-		renderer = theApp.GetConfig("renderer", 0);
+		renderer = static_cast<GSRendererType>(theApp.GetConfig("Renderer", static_cast<int>(GSRendererType::Default)));
 	}
 
 	if(threads == -1)
 	{
-		threads = theApp.GetConfig("extrathreads", 0);
+		threads = theApp.GetConfig("extrathreads", DEFAULT_EXTRA_RENDERING_THREADS);
 	}
 
-	GSWnd* wnd[2];
+	GSWnd* wnd[2] = { NULL, NULL };
 
 	try
 	{
-		if(s_renderer != renderer)
+		if (s_renderer != renderer)
 		{
 			// Emulator has made a render change request, which requires a completely
 			// new s_gs -- if the emu doesn't save/restore the GS state across this
@@ -218,57 +208,119 @@ static int _GSopen(void** dsp, char* title, int renderer, int threads = -1)
 			s_gs = NULL;
 		}
 
-		switch(renderer)
-		{
+		const char* renderer_fullname = "";
+		const char* renderer_mode = "";
+
+		switch (renderer)
+		{		
+		case GSRendererType::DX9_SW:
+		case GSRendererType::DX1011_SW:
+		case GSRendererType::Null_SW:
+		case GSRendererType::OGL_SW:
+			renderer_mode = "(Software mode)";
+			break;
+		case GSRendererType::DX9_Null:
+		case GSRendererType::DX1011_Null:
+		case GSRendererType::Null_Null:
+			renderer_mode = "(Null mode)";
+			break;
+		case GSRendererType::DX9_OpenCL:
+		case GSRendererType::DX1011_OpenCL:
+		case GSRendererType::Null_OpenCL:
+		case GSRendererType::OGL_OpenCL:
+			renderer_mode = "(OpenCL)";
+			break;
 		default:
-#ifdef _WINDOWS
-		case 0: case 1: case 2: case 14:
-			dev = new GSDevice9(); 
-			break;
-		case 3: case 4: case 5: case 15:
-			dev = new GSDevice11(); 
-			break;
-#endif
-		case 9: case 10: case 11: case 16:
-			dev = new GSDeviceNull(); 
-			break;
-		case 12: case 13: case 17:
-			dev = new GSDeviceOGL(); 
+			renderer_mode = "(Hardware mode)";
 			break;
 		}
 
-		if(dev == NULL)
+		switch (renderer)
+		{
+		default:
+#ifdef _WINDOWS
+		case GSRendererType::DX9_HW:
+		case GSRendererType::DX9_SW:
+		case GSRendererType::DX9_Null:
+		case GSRendererType::DX9_OpenCL:
+			dev = new GSDevice9();
+			s_renderer_name = " D3D9";
+			renderer_fullname = "Direct3D9";
+			break;
+		case GSRendererType::DX1011_HW:
+		case GSRendererType::DX1011_SW:
+		case GSRendererType::DX1011_Null:
+		case GSRendererType::DX1011_OpenCL:
+			dev = new GSDevice11();
+			s_renderer_name = " D3D11";
+			renderer_fullname = "Direct3D11";
+			break;
+#endif
+		case GSRendererType::Null_HW:
+		case GSRendererType::Null_SW:
+		case GSRendererType::Null_Null:
+		case GSRendererType::Null_OpenCL:
+			dev = new GSDeviceNull();
+			s_renderer_name = " Null";
+			renderer_fullname = "Null";
+			break;
+		case GSRendererType::OGL_HW:
+		case GSRendererType::OGL_SW:
+		case GSRendererType::OGL_OpenCL:
+			dev = new GSDeviceOGL();
+			s_renderer_name = " OGL";
+			renderer_fullname = "OpenGL";
+			break;
+		}
+
+		printf("Current Renderer: %s %s\n", renderer_fullname, renderer_mode);
+
+		if (dev == NULL)
 		{
 			return -1;
 		}
 
-		if(s_gs == NULL)
+		if (s_gs == NULL)
 		{
-			switch(renderer)
+			switch (renderer)
 			{
 			default:
 #ifdef _WINDOWS
-			case 0:
+			case GSRendererType::DX9_HW:
 				s_gs = (GSRenderer*)new GSRendererDX9();
+				s_renderer_type = " HW";
 				break;
-			case 3: 
-				s_gs = (GSRenderer*)new GSRendererDX11(); 
+			case GSRendererType::DX1011_HW:
+				s_gs = (GSRenderer*)new GSRendererDX11();
+				s_renderer_type = " HW";
 				break;
 #endif
-			case 12: 
-				s_gs = (GSRenderer*)new GSRendererOGL(); 
+			case GSRendererType::OGL_HW:
+				s_gs = (GSRenderer*)new GSRendererOGL();
+				s_renderer_type = " HW";
 				break;
-			case 1: case 4: case 10: case 13:
+			case GSRendererType::DX9_SW:
+			case GSRendererType::DX1011_SW:
+			case GSRendererType::Null_SW:
+			case GSRendererType::OGL_SW:
 				s_gs = new GSRendererSW(threads);
+				s_renderer_type = " SW";
 				break;
-			case 2: case 5: case 11:
+			case GSRendererType::DX9_Null:
+			case GSRendererType::DX1011_Null:
+			case GSRendererType::Null_Null:
 				s_gs = new GSRendererNull();
+				s_renderer_type = " Null";
 				break;
-			case 14: case 15: case 16: case 17:
+			case GSRendererType::DX9_OpenCL:
+			case GSRendererType::DX1011_OpenCL:
+			case GSRendererType::Null_OpenCL:
+			case GSRendererType::OGL_OpenCL:
 #ifdef ENABLE_OPENCL
 				s_gs = new GSRendererCL();
+				s_renderer_type = " OCL";
 #else
-				printf("GSdx error: opencl is disabled\n");
+				printf("GSdx error: OpenCL is disabled\n");
 #endif
 				break;
 			}
@@ -281,9 +333,11 @@ static int _GSopen(void** dsp, char* title, int renderer, int threads = -1)
 		if (s_gs->m_wnd == NULL)
 		{
 #ifdef _WINDOWS
-			switch(renderer)
+			switch (renderer)
 			{
-			case 12: case 13: case 17:
+			case GSRendererType::OGL_HW:
+			case GSRendererType::OGL_SW:
+			case GSRendererType::OGL_OpenCL:
 				s_gs->m_wnd = new GSWndWGL();
 				break;
 			default:
@@ -291,16 +345,16 @@ static int _GSopen(void** dsp, char* title, int renderer, int threads = -1)
 				break;
 			}
 #else
-#ifdef ENABLE_GLES
-			wnd[0] = NULL;
+#ifdef EGL_SUPPORTED
+			wnd[0] = new GSWndEGL();
+			wnd[1] = new GSWndOGL();
 #else
 			wnd[0] = new GSWndOGL();
 #endif
-			wnd[1] = new GSWndEGL();
 #endif
 		}
 	}
-	catch(std::exception& ex)
+	catch (std::exception& ex)
 	{
 		// Allowing std exceptions to escape the scope of the plugin callstack could
 		// be problematic, because of differing typeids between DLL and EXE compilations.
@@ -370,7 +424,7 @@ static int _GSopen(void** dsp, char* title, int renderer, int threads = -1)
 		if (s_gs->m_wnd) {
 			// A window was already attached to s_gs so we also
 			// need to restore the window state (Attach)
-			s_gs->m_wnd->Attach((void*)((uint32*)(dsp)+1), false);
+			s_gs->m_wnd->Attach((void*)((uptr*)(dsp)+1), false);
 		} else {
 			// No window found, try to attach a GLX win and retry 
 			// with EGL win if failed.
@@ -379,7 +433,7 @@ static int _GSopen(void** dsp, char* title, int renderer, int threads = -1)
 				{
 					if (wnd[i] == NULL) continue;
 
-					wnd[i]->Attach((void*)((uint32*)(dsp)+1), false);
+					wnd[i]->Attach((void*)((uptr*)(dsp)+1), false);
 					s_gs->m_wnd = wnd[i];
 
 					if (i == 0) delete wnd[1];
@@ -393,14 +447,23 @@ static int _GSopen(void** dsp, char* title, int renderer, int threads = -1)
 				}
 			}
 		}
+#endif
+#ifdef _WINDOWS
+		try
+		{
+			s_gs->m_wnd->Attach(*dsp, false);
+		}
+		catch (GSDXRecoverableError)
+		{
+			s_gs->m_wnd->Detach();
+			delete s_gs->m_wnd;
+			s_gs->m_wnd = NULL;
+		}
+#endif
 		if (s_gs->m_wnd == NULL)
 		{
 			return -1;
 		}
-#endif
-#ifdef _WINDOWS
-		s_gs->m_wnd->Attach(*dsp, false);
-#endif
 	}
 
 	if(!s_gs->CreateDevice(dev))
@@ -413,46 +476,64 @@ static int _GSopen(void** dsp, char* title, int renderer, int threads = -1)
 
 		return -1;
 	}
+
+	if (renderer == GSRendererType::OGL_HW && theApp.GetConfig("debug_glsl_shader", 0) == 2) {
+		printf("GSdx: test OpenGL shader. Please wait...\n\n");
+		static_cast<GSDeviceOGL*>(s_gs->m_dev)->SelfShaderTest();
+		printf("\nGSdx: test OpenGL shader done. It will now exit\n");
+		return -1;
+	}
 	
 	return 0;
 }
 
 EXPORT_C_(int) GSopen2(void** dsp, uint32 flags)
 {
-#ifdef __linux__
-	// Use ogl renderer as default otherwise it crash at startup
-	// GSRenderOGL only GSDeviceOGL (not GSDeviceNULL)
-	int renderer = theApp.GetConfig("renderer", 12);
-#else
-	int renderer = theApp.GetConfig("renderer", 0);
-#endif
+	static bool stored_toggle_state = false;
+	bool toggle_state = !!(flags & 4);
 
-	if(flags & 4)
+	GSRendererType renderer = s_renderer;
+	// Fresh start up or config file changed
+	if (renderer == GSRendererType::Undefined)
 	{
-#ifdef _WINDOWS
-		int best_sw_renderer = GSUtil::CheckDirect3D11Level() >= D3D_FEATURE_LEVEL_10_0 ? 4 : 1; // dx11 / dx9 sw
+#ifdef _WIN32
+		GSRendererType default_renderer = GSUtil::CheckDirect3D11Level() >= D3D_FEATURE_LEVEL_10_0 ? GSRendererType::DX1011_HW : GSRendererType::DX9_HW;
+#else
+		GSRendererType default_renderer = GSRendererType::Default;
+#endif
+		renderer = static_cast<GSRendererType>(theApp.GetConfig("Renderer", static_cast<int>(default_renderer)));
+	}
+	else if (stored_toggle_state != toggle_state)
+	{
+#ifdef _WIN32
+		GSRendererType best_sw_renderer = GSUtil::CheckDirect3D11Level() >= D3D_FEATURE_LEVEL_10_0 ? GSRendererType::DX1011_SW : GSRendererType::DX9_SW;
+
+
+		switch (renderer) {
+			// Use alternative renderer (SW if currently using HW renderer, and vice versa, keeping the same API and API version)
+		case GSRendererType::DX9_SW: renderer = GSRendererType::DX9_HW; break;
+		case GSRendererType::DX9_HW: renderer = GSRendererType::DX9_SW; break;
+		case GSRendererType::DX1011_SW: renderer = GSRendererType::DX1011_HW; break;
+		case GSRendererType::DX1011_HW: renderer = GSRendererType::DX1011_SW; break;
+		case GSRendererType::OGL_SW: renderer = GSRendererType::OGL_HW; break;
+		case GSRendererType::OGL_HW: renderer = GSRendererType::OGL_SW; break;
+		default: renderer = best_sw_renderer; break;// If wasn't using one of the above mentioned ones, use best SW renderer.
 
-		switch(renderer){
-			// Use alternative renderer (SW if currently using HW renderer, and vice versa, keeping the same DX level)
-			case 1: renderer = 0; break; // DX9:  SW to HW
-			case 0: renderer = 1; break; // DX9:  HW to SW
-			case 4: renderer = 3; break; // DX11: SW to HW
-			case 3: renderer = 4; break; // DX11: HW to SW
-			case 13: renderer = 12; break; // OGL: SW to HW
-			case 12: renderer = 13; break; // OGL: HW to SW
-			default: renderer = best_sw_renderer; // If wasn't using DX (e.g. SDL), use best SW renderer.
 		}
 
 #endif
 #ifdef __linux__
 		switch(renderer) {
-			case 13: renderer = 12; break; // OGL: SW to HW
-			case 12: renderer = 13; break; // OGL: HW to SW
+			// Use alternative renderer (SW if currently using HW renderer, and vice versa)
+		case GSRendererType::OGL_SW: renderer = GSRendererType::OGL_HW; break;
+		case GSRendererType::OGL_HW: renderer = GSRendererType::OGL_SW; break;
+		default: renderer = GSRendererType::OGL_SW; break; // fallback to OGL SW
 		}
 #endif
 	}
+	stored_toggle_state = toggle_state;
 
-	int retval = _GSopen(dsp, NULL, renderer);
+	int retval = _GSopen(dsp, "", renderer);
 
 	if (s_gs != NULL)
 		s_gs->SetAspectRatio(0);	 // PCSX2 manages the aspect ratios
@@ -462,7 +543,7 @@ EXPORT_C_(int) GSopen2(void** dsp, uint32 flags)
 	return retval;
 }
 
-EXPORT_C_(int) GSopen(void** dsp, char* title, int mt)
+EXPORT_C_(int) GSopen(void** dsp, const char* title, int mt)
 {
 	/*
 	if(!XInitThreads()) return -1;
@@ -472,7 +553,7 @@ EXPORT_C_(int) GSopen(void** dsp, char* title, int mt)
 	XCloseDisplay(display);
 	*/
 
-	int renderer = 0;
+	GSRendererType renderer = GSRendererType::Default;
 
 	// Legacy GUI expects to acquire vsync from the configuration files.
 
@@ -484,7 +565,7 @@ EXPORT_C_(int) GSopen(void** dsp, char* title, int mt)
 
 #ifdef _WINDOWS
 
-		renderer = GSUtil::CheckDirect3D11Level() >= D3D_FEATURE_LEVEL_10_0 ? 4 : 1; // dx11 / dx9 sw
+		renderer = GSUtil::CheckDirect3D11Level() >= D3D_FEATURE_LEVEL_10_0 ? GSRendererType::DX1011_SW : GSRendererType::DX9_SW;
 
 #endif
 
@@ -494,7 +575,7 @@ EXPORT_C_(int) GSopen(void** dsp, char* title, int mt)
 	{
 		// normal init
 
-		renderer = theApp.GetConfig("renderer", 0);
+		renderer = static_cast<GSRendererType>(theApp.GetConfig("Renderer", static_cast<int>(GSRendererType::Default)));
 	}
 
 	*dsp = NULL;
@@ -546,6 +627,7 @@ EXPORT_C GSwriteCSR(uint32 csr)
 
 EXPORT_C GSinitReadFIFO(uint8* mem)
 {
+	GL_PERF("Init Read FIFO1");
 	try
 	{
 		s_gs->InitReadFIFO(mem, 1);
@@ -568,6 +650,7 @@ EXPORT_C GSreadFIFO(uint8* mem)
 
 EXPORT_C GSinitReadFIFO2(uint8* mem, uint32 size)
 {
+	GL_PERF("Init Read FIFO2");
 	try
 	{
 		s_gs->InitReadFIFO(mem, size);
@@ -724,8 +807,8 @@ EXPORT_C GSconfigure()
 		if(!GSUtil::CheckSSE()) return;
 
 #ifdef _WINDOWS
-
-		if(GSSettingsDlg(s_isgsopen2).DoModal() == IDOK)
+		GSDialog::InitCommonControls();
+		if(GSSettingsDlg().DoModal() == IDOK)
 		{
 			if(s_gs != NULL && s_gs->m_wnd->IsManaged())
 			{
@@ -733,15 +816,20 @@ EXPORT_C GSconfigure()
 
 				GSshutdown();
 			}
+			// Force a reload of the gs state
+			s_renderer = GSRendererType::Undefined;
 		}
 
 #else
 
 		if (RunLinuxDialog()) {
 			theApp.ReloadConfig();
+			// Force a reload of the gs state
+			s_renderer = GSRendererType::Undefined;
 		}
 
 #endif
+
 	} catch (GSDXRecoverableError)
 	{
 	}
@@ -812,12 +900,22 @@ EXPORT_C_(int) GSsetupRecording(int start, void* data)
 		printf("GSdx: no s_gs for recording\n");
 		return 0;
 	}
+#ifdef __linux__
+	if (theApp.GetConfig("capture_enabled", 0)) {
+		printf("GSdx: Recording is disabled\n");
+		return 0;
+	}
+#endif
 
 	if(start & 1)
 	{
 		printf("GSdx: Recording start command\n");
-		if( s_gs->BeginCapture() )
+		if (s_gs->BeginCapture()) {
 			pt(" - Capture started\n");
+		} else {
+			pt(" - Capture cancelled\n");
+			return 0;
+		}
 	}
 	else
 	{
@@ -841,21 +939,15 @@ EXPORT_C GSgetLastTag(uint32* tag)
 
 EXPORT_C GSgetTitleInfo2(char* dest, size_t length)
 {
-	if (gsopen_done == false) {
-		//printf("GSdx: GSgetTitleInfo but GSOpen not yet done. Ignoring\n");
-		return;
-	}
-
 	string s = "GSdx";
+	s.append(s_renderer_name).append(s_renderer_type);
 
 	// TODO: this gets called from a different thread concurrently with GSOpen (on linux)
-	if(s_gs == NULL) return;
-
-	if(s_gs->m_GStitleInfoBuffer[0])
+	if (gsopen_done && s_gs != NULL && s_gs->m_GStitleInfoBuffer[0])
 	{
-		GSAutoLock lock(&s_gs->m_pGSsetTitle_Crit);
+		std::lock_guard<std::mutex> lock(s_gs->m_pGSsetTitle_Crit);
 
-		s = format("GSdx | %s", s_gs->m_GStitleInfoBuffer);
+		s.append(" | ").append(s_gs->m_GStitleInfoBuffer);
 
 		if(s.size() > length - 1)
 		{
@@ -976,13 +1068,13 @@ public:
 
 EXPORT_C GSReplay(HWND hwnd, HINSTANCE hinst, LPSTR lpszCmdLine, int nCmdShow)
 {
-	int renderer = -1;
+	GSRendererType renderer = GSRendererType::Undefined;
 
 	{
 		char* start = lpszCmdLine;
 		char* end = NULL;
 		long n = strtol(lpszCmdLine, &end, 10);
-		if(end > start) {renderer = n; lpszCmdLine = end;}
+		if(end > start) {renderer = static_cast<GSRendererType>(n); lpszCmdLine = end;}
 	}
 
 	while(*lpszCmdLine == ' ') lpszCmdLine++;
@@ -1338,13 +1430,13 @@ EXPORT_C GSBenchmark(HWND hwnd, HINSTANCE hinst, LPSTR lpszCmdLine, int nCmdShow
 
 				printf("%6d %6d | ", (int)((float)trlen * n / (end - start) / 1000), (int)((float)(w * h) * n / (end - start) / 1000));
 
-				const GSOffset* o = mem->GetOffset(TEX0.TBP0, TEX0.TBW, TEX0.PSM);
+				const GSOffset* off = mem->GetOffset(TEX0.TBP0, TEX0.TBW, TEX0.PSM);
 
 				start = clock();
 
 				for(int j = 0; j < n; j++)
 				{
-					(mem->*rtx)(o, r, ptr, w * 4, TEXA);
+					(mem->*rtx)(off, r, ptr, w * 4, TEXA);
 				}
 
 				end = clock();
@@ -1357,7 +1449,7 @@ EXPORT_C GSBenchmark(HWND hwnd, HINSTANCE hinst, LPSTR lpszCmdLine, int nCmdShow
 
 					for(int j = 0; j < n; j++)
 					{
-						(mem->*rtxP)(o, r, ptr, w, TEXA);
+						(mem->*rtxP)(off, r, ptr, w, TEXA);
 					}
 
 					end = clock();
@@ -1427,6 +1519,7 @@ EXPORT_C GSBenchmark(HWND hwnd, HINSTANCE hinst, LPSTR lpszCmdLine, int nCmdShow
 
 #include <sys/time.h>
 #include <sys/timeb.h>	// ftime(), struct timeb
+#include "GSLzma.h"
 
 inline unsigned long timeGetTime()
 {
@@ -1436,142 +1529,119 @@ inline unsigned long timeGetTime()
 	return (unsigned long)(t.time*1000 + t.millitm);
 }
 
-void _fread(void *ptr, size_t size, size_t nmemb, FILE *stream)
-{
-	static uint32 read_cnt = 0;
-	read_cnt++;
-
-	size_t result = fread(ptr, size, nmemb, stream);
-	if (result != nmemb) {
-		fprintf(stderr, "Read error\n");
-		exit(read_cnt);
-	}
-}
-
 // Note
 EXPORT_C GSReplay(char* lpszCmdLine, int renderer)
 {
 	GLLoader::in_replayer = true;
 
-// lpszCmdLine:
-//   First parameter is the renderer.
-//   Second parameter is the gs file to load and run.
-
-//EXPORT_C GSReplay(HWND hwnd, HINSTANCE hinst, LPSTR lpszCmdLine, int nCmdShow)
-#if 0
-	int renderer = -1;
+	GSRendererType m_renderer;
+	// Allow to easyly switch between SW/HW renderer -> this effectively removes the ability to select the renderer by function args
+	m_renderer = static_cast<GSRendererType>(theApp.GetConfig("Renderer", static_cast<int>(GSRendererType::Default)));
+	// alternatively:
+	// m_renderer = static_cast<GSRendererType>(renderer);
 
+	if (m_renderer != GSRendererType::OGL_HW && m_renderer != GSRendererType::OGL_SW)
 	{
-		char* start = lpszCmdLine;
-		char* end = NULL;
-		long n = strtol(lpszCmdLine, &end, 10);
-		if(end > start) {renderer = n; lpszCmdLine = end;}
-	}
-
-	while(*lpszCmdLine == ' ') lpszCmdLine++;
-
-	::SetPriorityClass(::GetCurrentProcess(), HIGH_PRIORITY_CLASS);
-#endif
-	// Allow to easyly switch between SW/HW renderer
-	renderer = theApp.GetConfig("renderer", 12);
-	if (renderer != 12 && renderer != 13)
-	{
-		fprintf(stderr, "wrong renderer selected %d\n", renderer);
+		fprintf(stderr, "wrong renderer selected %d\n", static_cast<int>(m_renderer));
 		return;
 	}
 
+	struct Packet {uint8 type, param; uint32 size, addr; vector<uint8> buff;};
+
+	list<Packet*> packets;
+	vector<uint8> buff;
 	vector<float> stats;
 	stats.clear();
+	uint8 regs[0x2000];
 
-	if(FILE* fp = fopen(lpszCmdLine, "rb"))
-	{
-		//Console console("GSdx", true);
+	GSinit();
 
-		GSinit();
+	GSsetBaseMem(regs);
 
-		uint8 regs[0x2000];
-		GSsetBaseMem(regs);
+	s_vsync = !!theApp.GetConfig("vsync", 0);
 
-		s_vsync = !!theApp.GetConfig("vsync", 0);
+	void* hWnd = NULL;
 
-		void* hWnd = NULL;
+	int err = _GSopen((void**)&hWnd, "", m_renderer);
+	if (err != 0) {
+		fprintf(stderr, "Error failed to GSopen\n");
+		return;
+	}
+	if (s_gs->m_wnd == NULL) return;
 
-		int err = _GSopen((void**)&hWnd, "", renderer);
-		if (err != 0) {
-			fprintf(stderr, "Error failed to GSopen\n");
-			return;
-		}
-		if (s_gs->m_wnd == NULL) return;
+	{ // Read .gs content
+		std::string f(lpszCmdLine);
+#ifdef LZMA_SUPPORTED
+		GSDumpFile* file = (f.size() >= 4) && (f.compare(f.size()-3, 3, ".xz") == 0)
+			? (GSDumpFile*) new GSDumpLzma(lpszCmdLine)
+			: (GSDumpFile*) new GSDumpRaw(lpszCmdLine);
+#else
+		GSDumpFile* file = new GSDumpRaw(lpszCmdLine);
+#endif
 
 		uint32 crc;
-		_fread(&crc, 4, 1, fp);
+		file->Read(&crc, 4);
 		GSsetGameCRC(crc, 0);
 
 		GSFreezeData fd;
-		_fread(&fd.size, 4, 1, fp);
+		file->Read(&fd.size, 4);
 		fd.data = new uint8[fd.size];
-		_fread(fd.data, fd.size, 1, fp);
+		file->Read(fd.data, fd.size);
+
 		GSfreeze(FREEZE_LOAD, &fd);
 		delete [] fd.data;
 
-		_fread(regs, 0x2000, 1, fp);
+		file->Read(regs, 0x2000);
 
 		GSvsync(1);
 
-		struct Packet {uint8 type, param; uint32 size, addr; vector<uint8> buff;};
 
-		list<Packet*> packets;
-		vector<uint8> buff;
-		int type;
-
-		while((type = fgetc(fp)) != EOF)
+		while(!file->IsEof())
 		{
+			uint8 type;
+			file->Read(&type, 1);
+
 			Packet* p = new Packet();
 
-			p->type = (uint8)type;
+			p->type = type;
 
 			switch(type)
 			{
 			case 0:
-
-				p->param = (uint8)fgetc(fp);
-
-				_fread(&p->size, 4, 1, fp);
+				file->Read(&p->param, 1);
+				file->Read(&p->size, 4);
 
 				switch(p->param)
 				{
 				case 0:
 					p->buff.resize(0x4000);
 					p->addr = 0x4000 - p->size;
-					_fread(&p->buff[p->addr], p->size, 1, fp);
+					file->Read(&p->buff[p->addr], p->size);
 					break;
 				case 1:
 				case 2:
 				case 3:
 					p->buff.resize(p->size);
-					_fread(&p->buff[0], p->size, 1, fp);
+					file->Read(&p->buff[0], p->size);
 					break;
 				}
 
 				break;
 
 			case 1:
-
-				p->param = (uint8)fgetc(fp);
+				file->Read(&p->param, 1);
 
 				break;
 
 			case 2:
-
-				_fread(&p->size, 4, 1, fp);
+				file->Read(&p->size, 4);
 
 				break;
 
 			case 3:
-
 				p->buff.resize(0x2000);
 
-				_fread(&p->buff[0], 0x2000, 1, fp);
+				file->Read(&p->buff[0], 0x2000);
 
 				break;
 			}
@@ -1579,30 +1649,38 @@ EXPORT_C GSReplay(char* lpszCmdLine, int renderer)
 			packets.push_back(p);
 		}
 
-		sleep(1);
+		delete file;
+	}
 
-		//while(IsWindowVisible(hWnd))
-		//FIXME map?
-		int finished = theApp.GetConfig("linux_replay", 1);
-		unsigned long frame_number = 0;
-		while(finished > 0)
+	sleep(1);
+
+	//while(IsWindowVisible(hWnd))
+	//FIXME map?
+	int finished = theApp.GetConfig("linux_replay", 1);
+	if (theApp.GetConfig("dump", 0)) {
+		fprintf(stderr, "Dump is enabled. Replay will be disabled\n");
+		finished = 1;
+	}
+	unsigned long frame_number = 0;
+	unsigned long total_frame_nb = 0;
+	while(finished > 0)
+	{
+		frame_number = 0;
+		unsigned long start = timeGetTime();
+		for(auto i = packets.begin(); i != packets.end(); i++)
 		{
-			frame_number = 0;
-			unsigned long start = timeGetTime();
-			for(auto i = packets.begin(); i != packets.end(); i++)
-			{
-				Packet* p = *i;
+			Packet* p = *i;
 
-				switch(p->type)
-				{
+			switch(p->type)
+			{
 				case 0:
 
 					switch(p->param)
 					{
-					case 0: GSgifTransfer1(&p->buff[0], p->addr); break;
-					case 1: GSgifTransfer2(&p->buff[0], p->size / 16); break;
-					case 2: GSgifTransfer3(&p->buff[0], p->size / 16); break;
-					case 3: GSgifTransfer(&p->buff[0], p->size / 16); break;
+						case 0: GSgifTransfer1(&p->buff[0], p->addr); break;
+						case 1: GSgifTransfer2(&p->buff[0], p->size / 16); break;
+						case 2: GSgifTransfer3(&p->buff[0], p->size / 16); break;
+						case 3: GSgifTransfer(&p->buff[0], p->size / 16); break;
 					}
 
 					break;
@@ -1627,60 +1705,69 @@ EXPORT_C GSReplay(char* lpszCmdLine, int renderer)
 					memcpy(regs, &p->buff[0], 0x2000);
 
 					break;
-				}
 			}
+		}
+
+		// Ensure the rendering is complete to measure correctly the time.
+		glFinish();
+
+		if (finished > 90) {
+			sleep(1);
+		} else {
 			unsigned long end = timeGetTime();
+			frame_number = std::max(1ul, frame_number); // avoid a potential division by 0
+
 			fprintf(stderr, "The %ld frames of the scene was render on %ldms\n", frame_number, end - start);
 			fprintf(stderr, "A means of %fms by frame\n", (float)(end - start)/(float)frame_number);
 
 			stats.push_back((float)(end - start));
 
-
-			sleep(1);
 			finished--;
+			total_frame_nb += frame_number;
 		}
+	}
 
-		if (theApp.GetConfig("linux_replay", 1) > 1) {
-			// Print some nice stats
-			// Skip first frame (shader compilation populate the result)
-			// it divides by 10 the standard deviation...
-			float n = (float)theApp.GetConfig("linux_replay", 1) - 1.0f;
-			float mean = 0;
-			float sd = 0;
-			for (auto i = stats.begin()+1; i != stats.end(); i++) {
-				mean += *i;
-			}
-			mean = mean/n;
-			for (auto i = stats.begin()+1; i != stats.end(); i++) {
-				sd += pow((*i)-mean, 2);
-			}
-			sd = sqrt(sd/n);
-
-			fprintf(stderr, "\n\nMean: %fms\n", mean);
-			fprintf(stderr, "Standard deviation: %fms\n", sd);
-			fprintf(stderr, "Mean by frame: %fms (%ffps)\n", mean/(float)frame_number, 1000.0f*frame_number/mean);
-			fprintf(stderr, "Standard deviatin by frame: %fms\n", sd/(float)frame_number);
+	if (theApp.GetConfig("linux_replay", 1) > 1) {
+		// Print some nice stats
+		// Skip first frame (shader compilation populate the result)
+		// it divides by 10 the standard deviation...
+		float n = (float)theApp.GetConfig("linux_replay", 1) - 1.0f;
+		float mean = 0;
+		float sd = 0;
+		for (auto i = stats.begin()+1; i != stats.end(); i++) {
+			mean += *i;
 		}
+		mean = mean/n;
+		for (auto i = stats.begin()+1; i != stats.end(); i++) {
+			sd += pow((*i)-mean, 2);
+		}
+		sd = sqrt(sd/n);
+
+		fprintf(stderr, "\n\nMean: %fms\n", mean);
+		fprintf(stderr, "Standard deviation: %fms\n", sd);
+		fprintf(stderr, "Mean by frame: %fms (%ffps)\n", mean/(float)frame_number, 1000.0f*frame_number/mean);
+		fprintf(stderr, "Standard deviatin by frame: %fms\n", sd/(float)frame_number);
+	}
 #ifdef ENABLE_OGL_DEBUG_MEM_BW
-		fprintf(stderr, "memory bandwith. T: %f. V: %f\n", (float)g_texture_upload_byte/(float)frame_number/1024, (float)g_vertex_upload_byte/(float)frame_number/1024);
+	total_frame_nb *= 1024;
+	fprintf(stderr, "memory bandwith. T: %f KB/f. V: %f KB/f. U: %f KB/f\n",
+			(float)g_real_texture_upload_byte/(float)total_frame_nb,
+			(float)g_vertex_upload_byte/(float)total_frame_nb,
+			(float)g_uniform_upload_byte/(float)total_frame_nb
+		   );
 #endif
 
-		for(auto i = packets.begin(); i != packets.end(); i++)
-		{
-			delete *i;
-		}
-
-		packets.clear();
-
-		sleep(1);
-
-		GSclose();
-		GSshutdown();
-
-		fclose(fp);
-	} else {
-		fprintf(stderr, "failed to open %s\n", lpszCmdLine);
+	for(auto i = packets.begin(); i != packets.end(); i++)
+	{
+		delete *i;
 	}
+
+	packets.clear();
+
+	sleep(1);
+
+	GSclose();
+	GSshutdown();
 }
 #endif
 
