@@ -22,29 +22,37 @@
 #pragma once
 
 #include "GSVector.h"
-#include "GSThread.h"
+#include "GSPng.h"
 
 #ifdef _WINDOWS
 #include "GSCaptureDlg.h"
 #endif
 
-class GSCapture : protected GSCritSec
+class GSCapture
 {
+	std::recursive_mutex m_lock;
 	bool m_capturing;
 	GSVector2i m_size;
+	uint64 m_frame;
+	std::string m_out_dir;
+	int m_threads;
 
 	#ifdef _WINDOWS
 
 	CComPtr<IGraphBuilder> m_graph;
 	CComPtr<IBaseFilter> m_src;
 
+	#elif __linux__
+
+	vector<GSPng::Worker*> m_workers;
+
 	#endif
 
 public:
 	GSCapture();
 	virtual ~GSCapture();
 
-	bool BeginCapture(float fps);
+	bool BeginCapture(float fps, GSVector2i recomendedResolution, float aspect);
 	bool DeliverFrame(const void* bits, int pitch, bool rgba);
 	bool EndCapture();
 
