@@ -23,6 +23,7 @@
 #include "GSdx.h"
 #include "GSDevice9.h"
 #include "resource.h"
+#include <fstream>
 
 GSDevice9::GSDevice9()
 	: m_lost(false)
@@ -84,9 +85,6 @@ static void FindAdapter(IDirect3D9 *d3d9, UINT &adapter, D3DDEVTYPE &devtype, st
 
 // if supported and null != msaa_desc, msaa_desc will contain requested Count and Quality
 
-D3DTEXTUREFILTERTYPE LinearToAnisotropic = !!theApp.GetConfig("AnisotropicFiltering", 0) && !theApp.GetConfig("paltex", 0) ? D3DTEXF_ANISOTROPIC : D3DTEXF_LINEAR;
-D3DTEXTUREFILTERTYPE PointToAnisotropic = !!theApp.GetConfig("AnisotropicFiltering", 0) && !theApp.GetConfig("paltex", 0) ? D3DTEXF_ANISOTROPIC : D3DTEXF_POINT;
-
 static bool IsMsaaSupported(IDirect3D9* d3d, UINT adapter, D3DDEVTYPE devtype, D3DFORMAT depth_format, uint32 msaaCount, DXGI_SAMPLE_DESC* msaa_desc = NULL)
 {
 	if(msaaCount > 16) return false;
@@ -292,11 +290,13 @@ bool GSDevice9::Create(GSWnd* wnd)
 		D3DDECL_END()
 	};
 
-	CompileShader(IDR_CONVERT_FX, "vs_main", NULL, &m_convert.vs, il_convert, countof(il_convert), &m_convert.il);
+	vector<unsigned char> shader;
+	theApp.LoadResource(IDR_CONVERT_FX, shader);
+	CompileShader((const char *)shader.data(), shader.size(), "vs_main", NULL, &m_convert.vs, il_convert, countof(il_convert), &m_convert.il);
 
 	for(size_t i = 0; i < countof(m_convert.ps); i++)
 	{
-		CompileShader(IDR_CONVERT_FX, format("ps_main%d", i), NULL, &m_convert.ps[i]);
+		CompileShader((const char *)shader.data(), shader.size(), format("ps_main%d", i), NULL, &m_convert.ps[i]);
 	}
 
 	m_convert.dss.DepthEnable = false;
@@ -304,6 +304,8 @@ bool GSDevice9::Create(GSWnd* wnd)
 
 	m_convert.bs.BlendEnable = false;
 	m_convert.bs.RenderTargetWriteMask = D3DCOLORWRITEENABLE_RGBA;
+	D3DTEXTUREFILTERTYPE LinearToAnisotropic = theApp.GetConfig("MaxAnisotropy", 0) && !theApp.GetConfig("paltex", 0) ? D3DTEXF_ANISOTROPIC : D3DTEXF_LINEAR;
+	D3DTEXTUREFILTERTYPE PointToAnisotropic = theApp.GetConfig("MaxAnisotropy", 0) && !theApp.GetConfig("paltex", 0) ? D3DTEXF_ANISOTROPIC : D3DTEXF_POINT;
 
 	m_convert.ln.FilterMin[0] = LinearToAnisotropic;
 	m_convert.ln.FilterMag[0] = LinearToAnisotropic;
@@ -323,9 +325,10 @@ bool GSDevice9::Create(GSWnd* wnd)
 
 	// merge
 
+	theApp.LoadResource(IDR_MERGE_FX, shader);
 	for(size_t i = 0; i < countof(m_merge.ps); i++)
 	{
-		CompileShader(IDR_MERGE_FX, format("ps_main%d", i), NULL, &m_merge.ps[i]);
+		CompileShader((const char *)shader.data(), shader.size(), format("ps_main%d", i), NULL, &m_merge.ps[i]);
 	}
 
 	m_merge.bs.BlendEnable = true;
@@ -339,9 +342,10 @@ bool GSDevice9::Create(GSWnd* wnd)
 
 	// interlace
 
+	theApp.LoadResource(IDR_INTERLACE_FX, shader);
 	for(size_t i = 0; i < countof(m_interlace.ps); i++)
 	{
-		CompileShader(IDR_INTERLACE_FX, format("ps_main%d", i), NULL, &m_interlace.ps[i]);
+		CompileShader((const char *)shader.data(), shader.size(), format("ps_main%d", i), NULL, &m_interlace.ps[i]);
 	}
 
 	// Shade Boost	
@@ -364,7 +368,8 @@ bool GSDevice9::Create(GSWnd* wnd)
 		{NULL, NULL},
 	};
 
-	CompileShader(IDR_SHADEBOOST_FX, "ps_main", macro, &m_shadeboost.ps);
+	theApp.LoadResource(IDR_SHADEBOOST_FX, shader);
+	CompileShader((const char *)shader.data(), shader.size(), "ps_main", macro, &m_shadeboost.ps);
 
 	// create shader layout
 
@@ -649,11 +654,13 @@ void GSDevice9::EndScene()
 
 void GSDevice9::ClearRenderTarget(GSTexture* t, const GSVector4& c)
 {
+	if (!t) return;
 	ClearRenderTarget(t, (c * 255 + 0.5f).zyxw().rgba32());
 }
 
 void GSDevice9::ClearRenderTarget(GSTexture* rt, uint32 c)
 {
+	if (!rt) return;
 	CComPtr<IDirect3DSurface9> surface;
 	m_dev->GetRenderTarget(0, &surface);
 	m_dev->SetRenderTarget(0, *(GSTexture9*)rt);
@@ -663,6 +670,7 @@ void GSDevice9::ClearRenderTarget(GSTexture* rt, uint32 c)
 
 void GSDevice9::ClearDepth(GSTexture* t, float c)
 {
+	if (!t) return;
 	CComPtr<IDirect3DSurface9> dssurface;
 	m_dev->GetDepthStencilSurface(&dssurface);
 	m_dev->SetDepthStencilSurface(*(GSTexture9*)t);
@@ -672,6 +680,7 @@ void GSDevice9::ClearDepth(GSTexture* t, float c)
 
 void GSDevice9::ClearStencil(GSTexture* t, uint8 c)
 {
+	if (!t) return;
 	CComPtr<IDirect3DSurface9> dssurface;
 	m_dev->GetDepthStencilSurface(&dssurface);
 	m_dev->SetDepthStencilSurface(*(GSTexture9*)t);
@@ -768,7 +777,7 @@ GSTexture* GSDevice9::Resolve(GSTexture* t)
 	return NULL;
 }
 
-GSTexture* GSDevice9::CopyOffscreen(GSTexture* src, const GSVector4& sr, int w, int h, int format)
+GSTexture* GSDevice9::CopyOffscreen(GSTexture* src, const GSVector4& sRect, int w, int h, int format, int ps_shader)
 {
 	GSTexture* dst = NULL;
 
@@ -786,11 +795,11 @@ GSTexture* GSDevice9::CopyOffscreen(GSTexture* src, const GSVector4& sr, int w,
 
 	if(GSTexture* rt = CreateRenderTarget(w, h, false, format))
 	{
-		GSVector4 dr(0, 0, w, h);
+		GSVector4 dRect(0, 0, w, h);
 
 		if(GSTexture* src2 = src->IsMSAA() ? Resolve(src) : src)
 		{
-			StretchRect(src2, sr, rt, dr, m_convert.ps[1], NULL, 0);
+			StretchRect(src2, sRect, rt, dRect, m_convert.ps[1], NULL, 0);
 
 			if(src2 != src) Recycle(src2);
 		}
@@ -808,30 +817,30 @@ GSTexture* GSDevice9::CopyOffscreen(GSTexture* src, const GSVector4& sr, int w,
 	return dst;
 }
 
-void GSDevice9::CopyRect(GSTexture* st, GSTexture* dt, const GSVector4i& r)
+void GSDevice9::CopyRect(GSTexture* sTex, GSTexture* dTex, const GSVector4i& r)
 {
-	if(!st || !dt)
+	if(!sTex || !dTex)
 	{
 		ASSERT(0);
 		return;
 	}
 
-	m_dev->StretchRect(*(GSTexture9*)st, r, *(GSTexture9*)dt, r, D3DTEXF_NONE);
+	m_dev->StretchRect(*(GSTexture9*)sTex, r, *(GSTexture9*)dTex, r, D3DTEXF_NONE);
 }
 
-void GSDevice9::StretchRect(GSTexture* st, const GSVector4& sr, GSTexture* dt, const GSVector4& dr, int shader, bool linear)
+void GSDevice9::StretchRect(GSTexture* sTex, const GSVector4& sRect, GSTexture* dTex, const GSVector4& dRect, int shader, bool linear)
 {
-	StretchRect(st, sr, dt, dr, m_convert.ps[shader], NULL, 0, linear);
+	StretchRect(sTex, sRect, dTex, dRect, m_convert.ps[shader], NULL, 0, linear);
 }
 
-void GSDevice9::StretchRect(GSTexture* st, const GSVector4& sr, GSTexture* dt, const GSVector4& dr, IDirect3DPixelShader9* ps, const float* ps_cb, int ps_cb_len, bool linear)
+void GSDevice9::StretchRect(GSTexture* sTex, const GSVector4& sRect, GSTexture* dTex, const GSVector4& dRect, IDirect3DPixelShader9* ps, const float* ps_cb, int ps_cb_len, bool linear)
 {
-	StretchRect(st, sr, dt, dr, ps, ps_cb, ps_cb_len, &m_convert.bs, linear);
+	StretchRect(sTex, sRect, dTex, dRect, ps, ps_cb, ps_cb_len, &m_convert.bs, linear);
 }
 
-void GSDevice9::StretchRect(GSTexture* st, const GSVector4& sr, GSTexture* dt, const GSVector4& dr, IDirect3DPixelShader9* ps, const float* ps_cb, int ps_cb_len, Direct3DBlendState9* bs, bool linear)
+void GSDevice9::StretchRect(GSTexture* sTex, const GSVector4& sRect, GSTexture* dTex, const GSVector4& dRect, IDirect3DPixelShader9* ps, const float* ps_cb, int ps_cb_len, Direct3DBlendState9* bs, bool linear)
 {
-	if(!st || !dt)
+	if(!sTex || !dTex)
 	{
 		ASSERT(0);
 		return;
@@ -839,27 +848,27 @@ void GSDevice9::StretchRect(GSTexture* st, const GSVector4& sr, GSTexture* dt, c
 
 	BeginScene();
 
-	GSVector2i ds = dt->GetSize();
+	GSVector2i ds = dTex->GetSize();
 
 	// om
 
 	OMSetDepthStencilState(&m_convert.dss);
 	OMSetBlendState(bs, 0);
-	OMSetRenderTargets(dt, NULL);
+	OMSetRenderTargets(dTex, NULL);
 
 	// ia
 
-	float left = dr.x * 2 / ds.x - 1.0f;
-	float top = 1.0f - dr.y * 2 / ds.y;
-	float right = dr.z * 2 / ds.x - 1.0f;
-	float bottom = 1.0f - dr.w * 2 / ds.y;
+	float left = dRect.x * 2 / ds.x - 1.0f;
+	float top = 1.0f - dRect.y * 2 / ds.y;
+	float right = dRect.z * 2 / ds.x - 1.0f;
+	float bottom = 1.0f - dRect.w * 2 / ds.y;
 
 	GSVertexPT1 vertices[] =
 	{
-		{GSVector4(left, top, 0.5f, 1.0f), GSVector2(sr.x, sr.y)},
-		{GSVector4(right, top, 0.5f, 1.0f), GSVector2(sr.z, sr.y)},
-		{GSVector4(left, bottom, 0.5f, 1.0f), GSVector2(sr.x, sr.w)},
-		{GSVector4(right, bottom, 0.5f, 1.0f), GSVector2(sr.z, sr.w)},
+		{GSVector4(left, top, 0.5f, 1.0f), GSVector2(sRect.x, sRect.y)},
+		{GSVector4(right, top, 0.5f, 1.0f), GSVector2(sRect.z, sRect.y)},
+		{GSVector4(left, bottom, 0.5f, 1.0f), GSVector2(sRect.x, sRect.w)},
+		{GSVector4(right, bottom, 0.5f, 1.0f), GSVector2(sRect.z, sRect.w)},
 	};
 
 	for(size_t i = 0; i < countof(vertices); i++)
@@ -879,7 +888,7 @@ void GSDevice9::StretchRect(GSTexture* st, const GSVector4& sr, GSTexture* dt, c
 	// ps
 
 	PSSetSamplerState(linear ? &m_convert.ln : &m_convert.pt);
-	PSSetShaderResources(st, NULL);
+	PSSetShaderResources(sTex, NULL);
 	PSSetShader(ps, ps_cb, ps_cb_len);
 
 	//
@@ -891,38 +900,38 @@ void GSDevice9::StretchRect(GSTexture* st, const GSVector4& sr, GSTexture* dt, c
 	EndScene();
 }
 
-void GSDevice9::DoMerge(GSTexture* st[2], GSVector4* sr, GSTexture* dt, GSVector4* dr, bool slbg, bool mmod, const GSVector4& c)
+void GSDevice9::DoMerge(GSTexture* sTex[2], GSVector4* sRect, GSTexture* dTex, GSVector4* dRect, bool slbg, bool mmod, const GSVector4& c)
 {
-	ClearRenderTarget(dt, c);
+	ClearRenderTarget(dTex, c);
 
-	if(st[1] && !slbg)
+	if(sTex[1] && !slbg)
 	{
-		StretchRect(st[1], sr[1], dt, dr[1], m_merge.ps[0], NULL, true);
+		StretchRect(sTex[1], sRect[1], dTex, dRect[1], m_merge.ps[0], NULL, true);
 	}
 
-	if(st[0])
+	if(sTex[0])
 	{
 		MergeConstantBuffer cb;
 
 		cb.BGColor = c;
 
-		StretchRect(st[0], sr[0], dt, dr[0], m_merge.ps[mmod ? 1 : 0], (const float*)&cb, 1, &m_merge.bs, true);
+		StretchRect(sTex[0], sRect[0], dTex, dRect[0], m_merge.ps[mmod ? 1 : 0], (const float*)&cb, 1, &m_merge.bs, true);
 	}
 }
 
-void GSDevice9::DoInterlace(GSTexture* st, GSTexture* dt, int shader, bool linear, float yoffset)
+void GSDevice9::DoInterlace(GSTexture* sTex, GSTexture* dTex, int shader, bool linear, float yoffset)
 {
-	GSVector4 s = GSVector4(dt->GetSize());
+	GSVector4 s = GSVector4(dTex->GetSize());
 
-	GSVector4 sr(0, 0, 1, 1);
-	GSVector4 dr(0.0f, yoffset, s.x, s.y + yoffset);
+	GSVector4 sRect(0, 0, 1, 1);
+	GSVector4 dRect(0.0f, yoffset, s.x, s.y + yoffset);
 
 	InterlaceConstantBuffer cb;
 
 	cb.ZrH = GSVector2(0, 1.0f / s.y);
 	cb.hH = (float)s.y / 2;
 
-	StretchRect(st, sr, dt, dr, m_interlace.ps[shader], (const float*)&cb, 1, linear);
+	StretchRect(sTex, sRect, dTex, dRect, m_interlace.ps[shader], (const float*)&cb, 1, linear);
 }
 
 void GSDevice9::InitExternalFX()
@@ -930,7 +939,25 @@ void GSDevice9::InitExternalFX()
 	if (!ExShader_Compiled)
 	{
 		try {
-			CompileShader("shaders/GSdx.fx", "ps_main", NULL, &m_shaderfx.ps);
+			std::string config_name(theApp.GetConfig("shaderfx_conf", "shaders/GSdx_FX_Settings.ini"));
+			std::ifstream fconfig(config_name);
+			std::stringstream shader;
+			if (fconfig.good())
+				shader << fconfig.rdbuf() << "\n";
+			else
+				fprintf(stderr, "GSdx: External shader config '%s' not loaded.\n", config_name.c_str());
+
+			std::string shader_name(theApp.GetConfig("shaderfx_glsl", "shaders/GSdx.fx"));
+			std::ifstream fshader(shader_name);
+			if (fshader.good())
+			{
+				shader << fshader.rdbuf();
+				CompileShader(shader.str().c_str(), shader.str().length(), "ps_main", NULL, &m_shaderfx.ps);
+			}
+			else
+			{
+				fprintf(stderr, "GSdx: External shader '%s' not loaded and will be disabled!\n", shader_name.c_str());
+			}
 		}
 		catch (GSDXRecoverableError) {
 			printf("GSdx: failed to compile external post-processing shader. \n");
@@ -939,12 +966,12 @@ void GSDevice9::InitExternalFX()
 	}
 }
 
-void GSDevice9::DoExternalFX(GSTexture* st, GSTexture* dt)
+void GSDevice9::DoExternalFX(GSTexture* sTex, GSTexture* dTex)
 {
-	GSVector2i s = dt->GetSize();
+	GSVector2i s = dTex->GetSize();
 
-	GSVector4 sr(0, 0, 1, 1);
-	GSVector4 dr(0, 0, s.x, s.y);
+	GSVector4 sRect(0, 0, 1, 1);
+	GSVector4 dRect(0, 0, s.x, s.y);
 
 	ExternalFXConstantBuffer cb;
 	
@@ -954,7 +981,7 @@ void GSDevice9::DoExternalFX(GSTexture* st, GSTexture* dt)
 	cb.rcpFrame = GSVector4(1.0f / s.x, 1.0f / s.y, 0.0f, 0.0f);
 	cb.rcpFrameOpt = GSVector4::zero();
 
-	StretchRect(st, sr, dt, dr, m_shaderfx.ps, (const float*)&cb, 2, true);
+	StretchRect(sTex, sRect, dTex, dRect, m_shaderfx.ps, (const float*)&cb, 2, true);
 }
 
 void GSDevice9::InitFXAA()
@@ -962,7 +989,9 @@ void GSDevice9::InitFXAA()
 	if (!FXAA_Compiled)
 	{
 		try {
-			CompileShader(IDR_FXAA_FX, "ps_main", NULL, &m_fxaa.ps);
+			vector<unsigned char> shader;
+			theApp.LoadResource(IDR_FXAA_FX, shader);
+			CompileShader((const char *)shader.data(), shader.size(), "ps_main", NULL, &m_fxaa.ps);
 		}
 		catch (GSDXRecoverableError) {
 			printf("GSdx: Failed to compile fxaa shader.\n");
@@ -971,12 +1000,12 @@ void GSDevice9::InitFXAA()
 	}
 }
 
-void GSDevice9::DoFXAA(GSTexture* st, GSTexture* dt)
+void GSDevice9::DoFXAA(GSTexture* sTex, GSTexture* dTex)
 {
-	GSVector2i s = dt->GetSize();
+	GSVector2i s = dTex->GetSize();
 
-	GSVector4 sr(0, 0, 1, 1);
-	GSVector4 dr(0, 0, s.x, s.y);
+	GSVector4 sRect(0, 0, 1, 1);
+	GSVector4 dRect(0, 0, s.x, s.y);
 
 	FXAAConstantBuffer cb;
 
@@ -985,22 +1014,22 @@ void GSDevice9::DoFXAA(GSTexture* st, GSTexture* dt)
 	cb.rcpFrame = GSVector4(1.0f / s.x, 1.0f / s.y, 0.0f, 0.0f);
 	cb.rcpFrameOpt = GSVector4::zero();
 
-	StretchRect(st, sr, dt, dr, m_fxaa.ps, (const float*)&cb, 2, true);
+	StretchRect(sTex, sRect, dTex, dRect, m_fxaa.ps, (const float*)&cb, 2, true);
 }
 
-void GSDevice9::DoShadeBoost(GSTexture* st, GSTexture* dt)
+void GSDevice9::DoShadeBoost(GSTexture* sTex, GSTexture* dTex)
 {
-	GSVector2i s = dt->GetSize();
+	GSVector2i s = dTex->GetSize();
 
-	GSVector4 sr(0, 0, 1, 1);
-	GSVector4 dr(0, 0, s.x, s.y);
+	GSVector4 sRect(0, 0, 1, 1);
+	GSVector4 dRect(0, 0, s.x, s.y);
 
 	ShadeBoostConstantBuffer cb;
 
 	cb.rcpFrame = GSVector4(1.0f / s.x, 1.0f / s.y, 0.0f, 0.0f);
 	cb.rcpFrameOpt = GSVector4::zero();
 
-	StretchRect(st, sr, dt, dr, m_shadeboost.ps, (const float*)&cb, 1, true);
+	StretchRect(sTex, sRect, dTex, dRect, m_shadeboost.ps, (const float*)&cb, 1, true);
 }
 
 void GSDevice9::SetupDATE(GSTexture* rt, GSTexture* ds, const GSVertexPT1* vertices, bool datm)
@@ -1418,84 +1447,7 @@ void GSDevice9::OMSetRenderTargets(GSTexture* rt, GSTexture* ds, const GSVector4
 	}
 }
 
-void GSDevice9::CompileShader(const char* fn, const string& entry, const D3DXMACRO* macro, IDirect3DVertexShader9** vs, const D3DVERTEXELEMENT9* layout, int count, IDirect3DVertexDeclaration9** il)
-{
-    vector<D3DXMACRO> m;
-
-    PrepareShaderMacro(m, macro);
-
-    HRESULT hr;
-
-    CComPtr<ID3DXBuffer> shader, error;
-
-    hr = D3DXCompileShaderFromFile(fn, &m[0], NULL, entry.c_str(), m_shader.vs.c_str(), 0, &shader, &error, NULL);
-
-    if(SUCCEEDED(hr))
-    {
-        hr = m_dev->CreateVertexShader((DWORD*)shader->GetBufferPointer(), vs);
-    }
-    else if(error)
-    {
-        printf("%s\n", (const char*)error->GetBufferPointer());
-    }
-
-    ASSERT(SUCCEEDED(hr));
-
-    if(FAILED(hr))
-    {
-        throw GSDXRecoverableError();
-    }
-
-    hr = m_dev->CreateVertexDeclaration(layout, il);
-
-    if(FAILED(hr))
-    {
-        throw GSDXRecoverableError();
-    }
-}
-
-void GSDevice9::CompileShader(const char* fn, const string& entry, const D3DXMACRO* macro, IDirect3DPixelShader9** ps)
-{
-    uint32 flags = 0;
-
-    if(m_shader.level >= D3D_FEATURE_LEVEL_9_3)
-    {
-        flags |= D3DXSHADER_AVOID_FLOW_CONTROL;
-    }
-    else
-    {
-        flags |= D3DXSHADER_SKIPVALIDATION;
-    }
-
-    vector<D3DXMACRO> m;
-
-    PrepareShaderMacro(m, macro);
-
-    HRESULT hr;
-
-    CComPtr<ID3DXBuffer> shader, error;
-
-    hr = D3DXCompileShaderFromFile(fn, &m[0], NULL, entry.c_str(), m_shader.ps.c_str(), flags, &shader, &error, NULL);
-
-    if(SUCCEEDED(hr))
-    {
-        hr = m_dev->CreatePixelShader((DWORD*)shader->GetBufferPointer(), ps);
-    }
-    else if(error)
-    {
-        printf("%s\n", (const char*)error->GetBufferPointer());
-    }
-
-    ASSERT(SUCCEEDED(hr));
-
-    if(FAILED(hr))
-    {
-        throw GSDXRecoverableError();
-    }
-}
-
-
-void GSDevice9::CompileShader(uint32 id, const string& entry, const D3DXMACRO* macro, IDirect3DVertexShader9** vs, const D3DVERTEXELEMENT9* layout, int count, IDirect3DVertexDeclaration9** il)
+void GSDevice9::CompileShader(const char *source, size_t size, const string& entry, const D3DXMACRO* macro, IDirect3DVertexShader9** vs, const D3DVERTEXELEMENT9* layout, int count, IDirect3DVertexDeclaration9** il)
 {
 	vector<D3DXMACRO> m;
 
@@ -1505,7 +1457,7 @@ void GSDevice9::CompileShader(uint32 id, const string& entry, const D3DXMACRO* m
 
 	CComPtr<ID3DXBuffer> shader, error;
 
-	hr = D3DXCompileShaderFromResource(theApp.GetModuleHandle(), MAKEINTRESOURCE(id), &m[0], NULL, entry.c_str(), m_shader.vs.c_str(), 0, &shader, &error, NULL);
+	hr = D3DXCompileShader(source, size, &m[0], NULL, entry.c_str(), m_shader.vs.c_str(), 0, &shader, &error, NULL);
 
 	if(SUCCEEDED(hr))
 	{
@@ -1516,6 +1468,8 @@ void GSDevice9::CompileShader(uint32 id, const string& entry, const D3DXMACRO* m
 		printf("%s\n", (const char*)error->GetBufferPointer());
 	}
 
+	ASSERT(SUCCEEDED(hr));
+
 	if(FAILED(hr))
 	{
 		throw GSDXRecoverableError();
@@ -1529,7 +1483,7 @@ void GSDevice9::CompileShader(uint32 id, const string& entry, const D3DXMACRO* m
 	}
 }
 
-void GSDevice9::CompileShader(uint32 id, const string& entry, const D3DXMACRO* macro, IDirect3DPixelShader9** ps)
+void GSDevice9::CompileShader(const char *source, size_t size, const string& entry, const D3DXMACRO* macro, IDirect3DPixelShader9** ps)
 {
 	uint32 flags = 0;
 
@@ -1549,8 +1503,7 @@ void GSDevice9::CompileShader(uint32 id, const string& entry, const D3DXMACRO* m
 	HRESULT hr;
 
 	CComPtr<ID3DXBuffer> shader, error;
-
-	hr = D3DXCompileShaderFromResource(theApp.GetModuleHandle(), MAKEINTRESOURCE(id), &m[0], NULL, entry.c_str(), m_shader.ps.c_str(), flags, &shader, &error, NULL);
+	hr = D3DXCompileShader(source, size, &m[0], NULL, entry.c_str(), m_shader.ps.c_str(), flags, &shader, &error, NULL);
 
 	if(SUCCEEDED(hr))
 	{
@@ -1561,9 +1514,10 @@ void GSDevice9::CompileShader(uint32 id, const string& entry, const D3DXMACRO* m
 		printf("%s\n", (const char*)error->GetBufferPointer());
 	}
 
+	ASSERT(SUCCEEDED(hr));
+
 	if(FAILED(hr))
 	{
 		throw GSDXRecoverableError();
 	}
 }
-
