@@ -1,5 +1,5 @@
 /*
- *	Copyright (C) 2007-2009 Gabest
+ *	Copyright (C) 2007-2015 Gabest
  *	http://www.gabest.org
  *
  *  This Program is free software; you can redistribute it and/or modify
@@ -37,3 +37,40 @@ struct GSSetting
 		this->note = note;
 	}
 };
+
+const char* dialog_message(int ID, bool* updateText = NULL);
+
+#ifdef __linux__
+enum {
+	IDC_FILTER,
+	IDC_SKIPDRAWHACK,
+	IDC_SKIPDRAWHACKEDIT,
+	IDC_ALPHAHACK,
+	IDC_OFFSETHACK,
+	IDC_SPRITEHACK,
+	IDC_WILDHACK,
+	IDC_MSAACB,
+	IDC_ALPHASTENCIL,
+	IDC_CHECK_DISABLE_ALL_HACKS,
+	IDC_ALIGN_SPRITE,
+	IDC_ROUND_SPRITE,
+	IDC_TCOFFSETX,
+	IDC_TCOFFSETX2,
+	IDC_TCOFFSETY,
+	IDC_TCOFFSETY2,
+	IDC_PALTEX,
+	IDC_ACCURATE_BLEND_UNIT,
+	IDC_ACCURATE_DATE,
+	IDC_TC_DEPTH,
+	IDC_CRC_LEVEL,
+	IDC_AFCOMBO,
+	IDC_AA1,
+	IDC_SWTHREADS,
+	IDC_SWTHREADS_EDIT,
+	IDC_SHADEBOOST,
+	IDC_SHADER_FX,
+	IDC_FXAA,
+	IDC_MIPMAP,
+	IDC_PRELOAD_GS
+};
+#endif
