@@ -28,7 +28,7 @@
 class GSTextureCacheOGL : public GSTextureCache
 {
 protected:
-	int Get8bitFormat() { return GL_R8; /* TODO return DXGI_FORMAT_A8_UNORM;*/}
+	int Get8bitFormat() { return GL_R8;}
 
 	void Read(Target* t, const GSVector4i& r);
 
