@@ -18,7 +18,7 @@
 #include <windows.h>
 #include <windowsx.h>
 
-extern int GSOpenWindow(void *pDsp, char *Title);
+extern int GSOpenWindow(void *pDsp, const char *Title);
 extern void GSCloseWindow();
 extern void GSProcessMessages();
 extern void HandleKeyEvent(keyEvent *ev);
