@@ -860,7 +860,7 @@ void Mix()
 	spu2M_WriteFast( 0xA00 + OutPos, Ext.Right );
 	WaveDump::WriteCore( 0, CoreSrc_External, Ext );
 
-	ApplyVolume( Ext, Cores[1].ExtVol );
+	Ext = ApplyVolume( Ext, Cores[1].ExtVol );
 	StereoOut32 Out( Cores[1].Mix( VoiceData[1], InputData[1], Ext ) );
 
 	if( PlayMode & 8 )
